@@ -28,11 +28,15 @@ type responseCallback func(interface{})
 type pluginResolveCallback func(uint32, map[string]interface{}) []byte
 
 type activeBuild struct {
-	ctx           api.BuildContext
-	pluginResolve pluginResolveCallback
-	mutex         sync.Mutex
-	didGetRebuild bool
-	waitGroup     sync.WaitGroup
+	ctx              api.BuildContext
+	pluginResolve    pluginResolveCallback
+	mutex            sync.Mutex
+	disposeWaitGroup sync.WaitGroup // Allows "dispose" to wait for all active tasks
+
+	// These are guarded by the mutex
+	rebuildWaitGroup   *sync.WaitGroup // Allows "cancel" to wait for all active rebuilds (within mutex because "sync.WaitGroup" isn't thread-safe)
+	withinRebuildCount int
+	didGetCancel       bool
 }
 
 type serviceType struct {
@@ -245,7 +249,7 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 			ctx := build.ctx
 			pluginResolve := build.pluginResolve
 			if ctx != nil && pluginResolve != nil {
-				build.waitGroup.Add(1)
+				build.disposeWaitGroup.Add(1)
 			}
 			build.mutex.Unlock()
 			if pluginResolve != nil {
@@ -253,7 +257,7 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 				go func() {
 					defer service.keepAliveWaitGroup.Done()
 					if ctx != nil {
-						defer build.waitGroup.Done()
+						defer build.disposeWaitGroup.Done()
 					}
 					service.sendPacket(pluginResolve(p.id, request))
 				}()
@@ -273,16 +277,31 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 			build.mutex.Lock()
 			ctx := build.ctx
 			if ctx != nil {
-				build.didGetRebuild = true
-				build.waitGroup.Add(1)
+				build.withinRebuildCount++
+				if build.rebuildWaitGroup == nil {
+					build.rebuildWaitGroup = &sync.WaitGroup{}
+				}
+				build.rebuildWaitGroup.Add(1)
+				build.disposeWaitGroup.Add(1)
 			}
 			build.mutex.Unlock()
 			if ctx != nil {
 				service.keepAliveWaitGroup.Add(1)
 				go func() {
 					defer service.keepAliveWaitGroup.Done()
-					defer build.waitGroup.Done()
+					defer build.disposeWaitGroup.Done()
 					result := ctx.Rebuild()
+					build.mutex.Lock()
+					build.withinRebuildCount--
+					build.rebuildWaitGroup.Done()
+					if build.withinRebuildCount == 0 {
+						// Clear the cancel flag now that the last rebuild has finished
+						build.didGetCancel = false
+
+						// Clear this to avoid confusion with the next group of rebuilds
+						build.rebuildWaitGroup = nil
+					}
+					build.mutex.Unlock()
 					service.sendPacket(encodePacket(packet{
 						id: p.id,
 						value: map[string]interface{}{
@@ -307,14 +326,14 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 			build.mutex.Lock()
 			ctx := build.ctx
 			if ctx != nil {
-				build.waitGroup.Add(1)
+				build.disposeWaitGroup.Add(1)
 			}
 			build.mutex.Unlock()
 			if ctx != nil {
 				service.keepAliveWaitGroup.Add(1)
 				go func() {
 					defer service.keepAliveWaitGroup.Done()
-					defer build.waitGroup.Done()
+					defer build.disposeWaitGroup.Done()
 					if err := ctx.Watch(api.WatchOptions{}); err != nil {
 						service.sendPacket(encodeErrorPacket(p.id, err))
 					} else {
@@ -340,14 +359,14 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 			build.mutex.Lock()
 			ctx := build.ctx
 			if ctx != nil {
-				build.waitGroup.Add(1)
+				build.disposeWaitGroup.Add(1)
 			}
 			build.mutex.Unlock()
 			if ctx != nil {
 				service.keepAliveWaitGroup.Add(1)
 				go func() {
 					defer service.keepAliveWaitGroup.Done()
-					defer build.waitGroup.Done()
+					defer build.disposeWaitGroup.Done()
 					var options api.ServeOptions
 					if value, ok := request["host"]; ok {
 						options.Host = value.(string)
@@ -374,7 +393,7 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 							build.mutex.Lock()
 							ctx := build.ctx
 							if ctx != nil {
-								build.waitGroup.Add(1)
+								build.disposeWaitGroup.Add(1)
 							}
 							build.mutex.Unlock()
 							if ctx != nil {
@@ -389,7 +408,7 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 										"timeInMS":      args.TimeInMS,
 									},
 								})
-								build.waitGroup.Done()
+								build.disposeWaitGroup.Done()
 							}
 						}
 					}
@@ -415,6 +434,49 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 			},
 		}))
 
+	case "cancel":
+		key := request["key"].(int)
+		if build := service.getActiveBuild(key); build != nil {
+			build.mutex.Lock()
+			ctx := build.ctx
+			rebuildWaitGroup := build.rebuildWaitGroup
+			if build.withinRebuildCount > 0 {
+				// If Go got a "rebuild" message from JS before this, there's a chance
+				// that Go hasn't run "ctx.Rebuild()" by the time our "ctx.Cancel()"
+				// runs below because both of them are on separate goroutines. To
+				// handle this, we set this flag to tell our "OnStart" plugin to cancel
+				// the build in case things happen in that order.
+				build.didGetCancel = true
+			}
+			build.mutex.Unlock()
+			if ctx != nil {
+				service.keepAliveWaitGroup.Add(1)
+				go func() {
+					defer service.keepAliveWaitGroup.Done()
+					ctx.Cancel()
+
+					// Block until all manual rebuilds that were active at the time the
+					// "cancel" packet was originally processed have finished. That way
+					// JS can wait for "cancel" to end and be assured that it can call
+					// "rebuild" and have it not merge with any other ongoing rebuilds.
+					if rebuildWaitGroup != nil {
+						rebuildWaitGroup.Wait()
+					}
+
+					// Only return control to JavaScript once the cancel operation has succeeded
+					service.sendPacket(encodePacket(packet{
+						id:    p.id,
+						value: make(map[string]interface{}),
+					}))
+				}()
+				return
+			}
+		}
+		service.sendPacket(encodePacket(packet{
+			id:    p.id,
+			value: make(map[string]interface{}),
+		}))
+
 	case "dispose":
 		key := request["key"].(int)
 		if build := service.getActiveBuild(key); build != nil {
@@ -437,8 +499,8 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 					// wait for it here before disposing. Once the wait is over, no more
 					// operations can happen on the context because we have already
 					// zeroed out the shared context pointer above.
-					build.waitGroup.Done()
-					build.waitGroup.Wait()
+					build.disposeWaitGroup.Done()
+					build.disposeWaitGroup.Wait()
 
 					ctx.Dispose()
 					service.destroyActiveBuild(key)
@@ -597,30 +659,56 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 			Setup: func(build api.PluginBuild) {
 				build.OnStart(func() (api.OnStartResult, error) {
 					activeBuild.mutex.Lock()
-					activeBuild.didGetRebuild = true
+					if currentWaitGroup := activeBuild.rebuildWaitGroup; currentWaitGroup != nil && activeBuild.didGetCancel {
+						// Cancel the current build now that the current build is active.
+						// This catches the case where JS does "rebuild()" then "cancel()"
+						// but Go's scheduler runs the original "ctx.Cancel()" goroutine
+						// before it runs the "ctx.Rebuild()" goroutine.
+						//
+						// This adds to the rebuild wait group that other cancel operations
+						// are waiting on because we also want those other cancel operations
+						// to wait on this cancel operation. Go might schedule this new
+						// goroutine after all currently-active rebuilds end. We don't want
+						// the user's cancel operation to return to the user and for them
+						// to start another rebuild before our "ctx.Cancel" below runs
+						// because our cancel is supposed to cancel the current build, not
+						// some independent future build.
+						activeBuild.rebuildWaitGroup.Add(1)
+						go func() {
+							activeBuild.ctx.Cancel()
+
+							// Lock the mutex because "sync.WaitGroup" isn't thread-safe.
+							// But use the wait group that was active at the time the
+							// "OnStart" callback ran instead of the latest one on the
+							// active build in case this goroutine is delayed.
+							activeBuild.mutex.Lock()
+							currentWaitGroup.Done()
+							activeBuild.mutex.Unlock()
+						}()
+					}
 					activeBuild.mutex.Unlock()
 					return api.OnStartResult{}, nil
 				})
 
 				build.OnEnd(func(result *api.BuildResult) (api.OnEndResult, error) {
 					// For performance, we only send JavaScript an "onEnd" message if
-					// it's needed. It's only needed if there are any "onEnd" callbacks
-					// registered or if JavaScript has called our "rebuild()" function
-					// (since the result for "rebuild" is passed via the same mechanism).
+					// it's needed. It's only needed if one of the following is true:
+					//
+					// - There are any "onEnd" callbacks registered
+					// - JavaScript has called our "rebuild()" function
+					// - We are writing build output to JavaScript's stdout
+					//
 					// This is especially important if "write" is false since otherwise
 					// we'd unnecessarily send the entire contents of all output files!
 					//
 					//          "If a tree falls in a forest and no one is
 					//           around to hear it, does it make a sound?"
 					//
-					if !hasOnEndCallbacks {
-						activeBuild.mutex.Lock()
-						didGetRebuild := activeBuild.didGetRebuild
-						activeBuild.didGetRebuild = false
-						activeBuild.mutex.Unlock()
-						if !didGetRebuild {
-							return api.OnEndResult{}, nil
-						}
+					activeBuild.mutex.Lock()
+					isWithinRebuild := activeBuild.withinRebuildCount > 0
+					activeBuild.mutex.Unlock()
+					if !hasOnEndCallbacks && !isWithinRebuild && !writeToStdout {
+						return api.OnEndResult{}, nil
 					}
 					request := resultToResponse(*result)
 					request["command"] = "on-end"
@@ -657,7 +745,7 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 		}
 
 		// Keep the build alive until "dispose" has been called
-		activeBuild.waitGroup.Add(1)
+		activeBuild.disposeWaitGroup.Add(1)
 		activeBuild.ctx = ctx
 
 		return encodePacket(packet{
