@@ -700,18 +700,106 @@ func TestCSSExternalQueryAndHashMatchIssue1822(t *testing.T) {
 func TestCSSNestingOldBrowser(t *testing.T) {
 	css_suite.expectBundled(t, bundled{
 		files: map[string]string{
-			"/entry.css": `
-				a { &:hover { color: red; } }
-			`,
+			"/nested-@layer.css":          `a { @layer base { color: red; } }`,
+			"/nested-@media.css":          `a { @media screen { color: red; } }`,
+			"/nested-ampersand-twice.css": `a { &, & { color: red; } }`,
+			"/nested-ampersand-first.css": `a { &, b { color: red; } }`,
+			"/nested-attribute.css":       `a { [href] { color: red; } }`,
+			"/nested-colon.css":           `a { :hover { color: red; } }`,
+			"/nested-dot.css":             `a { .cls { color: red; } }`,
+			"/nested-greaterthan.css":     `a { > b { color: red; } }`,
+			"/nested-hash.css":            `a { #id { color: red; } }`,
+			"/nested-plus.css":            `a { + b { color: red; } }`,
+			"/nested-tilde.css":           `a { ~ b { color: red; } }`,
+
+			"/toplevel-ampersand-twice.css":  `&, & { color: red; }`,
+			"/toplevel-ampersand-first.css":  `&, a { color: red; }`,
+			"/toplevel-ampersand-second.css": `a, & { color: red; }`,
+			"/toplevel-attribute.css":        `[href] { color: red; }`,
+			"/toplevel-colon.css":            `:hover { color: red; }`,
+			"/toplevel-dot.css":              `.cls { color: red; }`,
+			"/toplevel-greaterthan.css":      `> b { color: red; }`,
+			"/toplevel-hash.css":             `#id { color: red; }`,
+			"/toplevel-plus.css":             `+ b { color: red; }`,
+			"/toplevel-tilde.css":            `~ b { color: red; }`,
+
+			"/media-ampersand-twice.css":  `@media screen { &, & { color: red; } }`,
+			"/media-ampersand-first.css":  `@media screen { &, a { color: red; } }`,
+			"/media-ampersand-second.css": `@media screen { a, & { color: red; } }`,
+			"/media-attribute.css":        `@media screen { [href] { color: red; } }`,
+			"/media-colon.css":            `@media screen { :hover { color: red; } }`,
+			"/media-dot.css":              `@media screen { .cls { color: red; } }`,
+			"/media-greaterthan.css":      `@media screen { > b { color: red; } }`,
+			"/media-hash.css":             `@media screen { #id { color: red; } }`,
+			"/media-plus.css":             `@media screen { + b { color: red; } }`,
+			"/media-tilde.css":            `@media screen { ~ b { color: red; } }`,
+		},
+		entryPaths: []string{
+			"/nested-@layer.css",
+			"/nested-@media.css",
+			"/nested-ampersand-twice.css",
+			"/nested-ampersand-first.css",
+			"/nested-attribute.css",
+			"/nested-colon.css",
+			"/nested-dot.css",
+			"/nested-greaterthan.css",
+			"/nested-hash.css",
+			"/nested-plus.css",
+			"/nested-tilde.css",
+
+			"/toplevel-ampersand-twice.css",
+			"/toplevel-ampersand-first.css",
+			"/toplevel-ampersand-second.css",
+			"/toplevel-attribute.css",
+			"/toplevel-colon.css",
+			"/toplevel-dot.css",
+			"/toplevel-greaterthan.css",
+			"/toplevel-hash.css",
+			"/toplevel-plus.css",
+			"/toplevel-tilde.css",
+
+			"/media-ampersand-twice.css",
+			"/media-ampersand-first.css",
+			"/media-ampersand-second.css",
+			"/media-attribute.css",
+			"/media-colon.css",
+			"/media-dot.css",
+			"/media-greaterthan.css",
+			"/media-hash.css",
+			"/media-plus.css",
+			"/media-tilde.css",
 		},
-		entryPaths: []string{"/entry.css"},
 		options: config.Options{
 			Mode:                   config.ModeBundle,
-			AbsOutputFile:          "/out.css",
-			UnsupportedCSSFeatures: compat.Nesting,
+			AbsOutputDir:           "/out",
+			UnsupportedCSSFeatures: compat.Nesting | compat.IsPseudoClass,
 			OriginalTargetEnv:      "chrome10",
 		},
-		expectedScanLog: `entry.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+		expectedScanLog: `media-ampersand-first.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-ampersand-second.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-ampersand-twice.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-ampersand-twice.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-greaterthan.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-plus.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+media-tilde.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-@layer.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-@media.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-ampersand-first.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-ampersand-twice.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-attribute.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-colon.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-dot.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-greaterthan.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-hash.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-plus.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+nested-tilde.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-ampersand-first.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-ampersand-second.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-ampersand-twice.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-ampersand-twice.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-greaterthan.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-plus.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+toplevel-tilde.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
 `,
 	})
 }
@@ -762,8 +850,9 @@ func TestDeduplicateRules(t *testing.T) {
 			"/yes0.css": "a { color: red; color: green; color: red }",
 			"/yes1.css": "a { color: red } a { color: green } a { color: red }",
 			"/yes2.css": "@media screen { a { color: red } } @media screen { a { color: red } }",
+			"/yes3.css": "@media screen { a { color: red } } @media screen { & a { color: red } }",
 
-			"/no0.css": "@media screen { a { color: red } } @media screen { & a { color: red } }",
+			"/no0.css": "@media screen { a { color: red } } @media screen { &a { color: red } }",
 			"/no1.css": "@media screen { a { color: red } } @media screen { a[x] { color: red } }",
 			"/no2.css": "@media screen { a { color: red } } @media screen { a.x { color: red } }",
 			"/no3.css": "@media screen { a { color: red } } @media screen { a#x { color: red } }",
@@ -775,11 +864,17 @@ func TestDeduplicateRules(t *testing.T) {
 			"/across-files-0.css": "a { color: red; color: red }",
 			"/across-files-1.css": "a { color: green }",
 			"/across-files-2.css": "a { color: red }",
+
+			"/across-files-url.css":   "@import 'across-files-url-0.css'; @import 'across-files-url-1.css'; @import 'across-files-url-2.css';",
+			"/across-files-url-0.css": "@import 'http://example.com/some.css'; @font-face { src: url(http://example.com/some.font); }",
+			"/across-files-url-1.css": "@font-face { src: url(http://example.com/some.other.font); }",
+			"/across-files-url-2.css": "@font-face { src: url(http://example.com/some.font); }",
 		},
 		entryPaths: []string{
 			"/yes0.css",
 			"/yes1.css",
 			"/yes2.css",
+			"/yes3.css",
 
 			"/no0.css",
 			"/no1.css",
@@ -790,12 +885,12 @@ func TestDeduplicateRules(t *testing.T) {
 			"/no6.css",
 
 			"/across-files.css",
+			"/across-files-url.css",
 		},
 		options: config.Options{
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
 			MinifySyntax: true,
 		},
-		expectedScanLog: "no0.css: WARNING: CSS nesting syntax cannot be used outside of a style rule\n",
 	})
 }
