@@ -185,10 +185,8 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "` +
-			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false ` +
-			`in the enclosing "package.json" file
+		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file:
 `,
 	})
 }
@@ -216,10 +214,8 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireCommonJS(t *testing.
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "` +
-			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false ` +
-			`in the enclosing "package.json" file
+		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file:
 `,
 	})
 }
@@ -246,10 +242,8 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "` +
-			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false ` +
-			`in the enclosing "package.json" file
+		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file:
 `,
 	})
 }
@@ -276,10 +270,8 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportCommonJS(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "` +
-			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false ` +
-			`in the enclosing "package.json" file
+		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file:
 `,
 	})
 }
@@ -724,10 +716,8 @@ func TestPackageJsonSideEffectsArrayGlob(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because ` +
-			`"Users/user/project/node_modules/demo-pkg/remove/this/file.js" was marked as having no side effects
-Users/user/project/node_modules/demo-pkg/package.json: NOTE: It was excluded from the "sideEffects" ` +
-			`array in the enclosing "package.json" file
+		expectedScanLog: `Users/user/project/src/entry.js: WARNING: Ignoring this import because "Users/user/project/node_modules/demo-pkg/remove/this/file.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: NOTE: It was excluded from the "sideEffects" array in the enclosing "package.json" file:
 `,
 	})
 }
@@ -3444,3 +3434,316 @@ func TestPackageJsonSideEffectsFalseCrossPlatformSlash(t *testing.T) {
 		},
 	})
 }
+
+func TestTreeShakingJSWithAssociatedCSS(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg/button'
+				import { Menu } from 'pkg/menu'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/button.js": `
+				import './button.css'
+				export let Button
+			`,
+			"/project/node_modules/pkg/button.css": `
+				button { color: red }
+			`,
+			"/project/node_modules/pkg/menu.js": `
+				import './menu.css'
+				export let Menu
+			`,
+			"/project/node_modules/pkg/menu.css": `
+				menu { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSReExportSideEffectsFalse(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/entry.js": `
+				export { Button } from './components'
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"main": "./entry.js",
+				"sideEffects": false
+			}`,
+			"/project/node_modules/pkg/components.jsx": `
+				require('./button.css')
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/button.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSReExportSideEffectsFalseOnlyJS(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/entry.js": `
+				export { Button } from './components'
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"main": "./entry.js",
+				"sideEffects": ["*.css"]
+			}`,
+			"/project/node_modules/pkg/components.jsx": `
+				require('./button.css')
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/button.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSExportStarSideEffectsFalse(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/entry.js": `
+				export * from './components'
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"main": "./entry.js",
+				"sideEffects": false
+			}`,
+			"/project/node_modules/pkg/components.jsx": `
+				require('./button.css')
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/button.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSExportStarSideEffectsFalseOnlyJS(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/entry.js": `
+				export * from './components'
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"main": "./entry.js",
+				"sideEffects": ["*.css"]
+			}`,
+			"/project/node_modules/pkg/components.jsx": `
+				require('./button.css')
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/button.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSUnusedNestedImportSideEffectsFalse(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg/button'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"sideEffects": false
+			}`,
+			"/project/node_modules/pkg/button.jsx": `
+				import styles from './styles'
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/styles.js": `
+				import './styles.css'
+				export default {}
+			`,
+			"/project/node_modules/pkg/styles.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTreeShakingJSWithAssociatedCSSUnusedNestedImportSideEffectsFalseOnlyJS(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/project/test.jsx": `
+				import { Button } from 'pkg/button'
+				render(<Button/>)
+			`,
+			"/project/node_modules/pkg/package.json": `{
+				"sideEffects": ["*.css"]
+			}`,
+			"/project/node_modules/pkg/button.jsx": `
+				import styles from './styles'
+				export const Button = () => <button/>
+			`,
+			"/project/node_modules/pkg/styles.js": `
+				import './styles.css'
+				export default {}
+			`,
+			"/project/node_modules/pkg/styles.css": `
+				button { color: red }
+			`,
+		},
+		entryPaths: []string{"/project/test.jsx"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestPreserveDirectivesMinifyPassThrough(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				//! 1
+				'use 1'
+				//! 2
+				'use 2'
+				//! 3
+				'use 3'
+				entry()
+				//! 4
+				'use 4'
+				//! 5
+				'use 5'
+				//! 6
+				'use 6'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
+
+func TestPreserveDirectivesMinifyIIFE(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				//! 1
+				'use 1'
+				//! 2
+				'use 2'
+				//! 3
+				'use 3'
+				entry()
+				//! 4
+				'use 4'
+				//! 5
+				'use 5'
+				//! 6
+				'use 6'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatIIFE,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
+
+func TestPreserveDirectivesMinifyBundle(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				//! 1
+				'use 1'
+				//! 2
+				'use 2'
+				//! 3
+				'use 3'
+				entry()
+				//! 4
+				'use 4'
+				//! 5
+				'use 5'
+				//! 6
+				'use 6'
+				import "./nested.js"
+			`,
+			"/nested.js": `
+				//! A
+				'use A'
+				//! B
+				'use B'
+				//! C
+				'use C'
+				nested()
+				//! D
+				'use D'
+				//! E
+				'use E'
+				//! F
+				'use F'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			OutputFormat:  config.FormatIIFE,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
