@@ -79,13 +79,42 @@ const (
 	WhitespaceAfter
 )
 
-func (a Token) Equal(b Token) bool {
-	if a.Kind == b.Kind && a.Text == b.Text && a.ImportRecordIndex == b.ImportRecordIndex && a.Whitespace == b.Whitespace {
+// This is necessary when comparing tokens between two different files
+type CrossFileEqualityCheck struct {
+	ImportRecordsA []ast.ImportRecord
+	ImportRecordsB []ast.ImportRecord
+}
+
+func (a Token) Equal(b Token, check *CrossFileEqualityCheck) bool {
+	if a.Kind == b.Kind && a.Text == b.Text && a.Whitespace == b.Whitespace {
+		// URLs should be compared based on the text of the associated import record
+		// (which is what will actually be printed) instead of the original text
+		if a.Kind == css_lexer.TURL {
+			if check == nil {
+				// If both tokens are in the same file, just compare the index
+				if a.ImportRecordIndex != b.ImportRecordIndex {
+					return false
+				}
+			} else {
+				// If the tokens come from separate files, compare the import records
+				// themselves instead of comparing the indices. This can happen when
+				// the linker runs a "DuplicateRuleRemover" during bundling. This
+				// doesn't compare the source indices because at this point during
+				// linking, paths inside the bundle (e.g. due to the "copy" loader)
+				// should have already been converted into text (e.g. the "unique key"
+				// string).
+				if check.ImportRecordsA[a.ImportRecordIndex].Path.Text !=
+					check.ImportRecordsB[b.ImportRecordIndex].Path.Text {
+					return false
+				}
+			}
+		}
+
 		if a.Children == nil && b.Children == nil {
 			return true
 		}
 
-		if a.Children != nil && b.Children != nil && TokensEqual(*a.Children, *b.Children) {
+		if a.Children != nil && b.Children != nil && TokensEqual(*a.Children, *b.Children, check) {
 			return true
 		}
 	}
@@ -93,12 +122,12 @@ func (a Token) Equal(b Token) bool {
 	return false
 }
 
-func TokensEqual(a []Token, b []Token) bool {
+func TokensEqual(a []Token, b []Token, check *CrossFileEqualityCheck) bool {
 	if len(a) != len(b) {
 		return false
 	}
-	for i, c := range a {
-		if !c.Equal(b[i]) {
+	for i, ai := range a {
+		if !ai.Equal(b[i], check) {
 			return false
 		}
 	}
@@ -110,7 +139,9 @@ func HashTokens(hash uint32, tokens []Token) uint32 {
 
 	for _, t := range tokens {
 		hash = helpers.HashCombine(hash, uint32(t.Kind))
-		hash = helpers.HashCombineString(hash, t.Text)
+		if t.Kind != css_lexer.TURL {
+			hash = helpers.HashCombineString(hash, t.Text)
+		}
 		if t.Children != nil {
 			hash = HashTokens(hash, *t.Children)
 		}
@@ -262,16 +293,16 @@ type Rule struct {
 }
 
 type R interface {
-	Equal(rule R) bool
+	Equal(rule R, check *CrossFileEqualityCheck) bool
 	Hash() (uint32, bool)
 }
 
-func RulesEqual(a []Rule, b []Rule) bool {
+func RulesEqual(a []Rule, b []Rule, check *CrossFileEqualityCheck) bool {
 	if len(a) != len(b) {
 		return false
 	}
-	for i, c := range a {
-		if !c.Data.Equal(b[i].Data) {
+	for i, ai := range a {
+		if !ai.Data.Equal(b[i].Data, check) {
 			return false
 		}
 	}
@@ -294,7 +325,7 @@ type RAtCharset struct {
 	Encoding string
 }
 
-func (a *RAtCharset) Equal(rule R) bool {
+func (a *RAtCharset) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RAtCharset)
 	return ok && a.Encoding == b.Encoding
 }
@@ -310,7 +341,7 @@ type RAtImport struct {
 	ImportRecordIndex uint32
 }
 
-func (*RAtImport) Equal(rule R) bool {
+func (*RAtImport) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	return false
 }
 
@@ -329,7 +360,7 @@ type KeyframeBlock struct {
 	Rules     []Rule
 }
 
-func (a *RAtKeyframes) Equal(rule R) bool {
+func (a *RAtKeyframes) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	if b, ok := rule.(*RAtKeyframes); ok && a.AtToken == b.AtToken && a.Name == b.Name && len(a.Blocks) == len(b.Blocks) {
 		for i, ai := range a.Blocks {
 			bi := b.Blocks[i]
@@ -341,7 +372,7 @@ func (a *RAtKeyframes) Equal(rule R) bool {
 					return false
 				}
 			}
-			if !RulesEqual(ai.Rules, bi.Rules) {
+			if !RulesEqual(ai.Rules, bi.Rules, check) {
 				return false
 			}
 		}
@@ -371,9 +402,9 @@ type RKnownAt struct {
 	Rules   []Rule
 }
 
-func (a *RKnownAt) Equal(rule R) bool {
+func (a *RKnownAt) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RKnownAt)
-	return ok && a.AtToken == b.AtToken && TokensEqual(a.Prelude, b.Prelude) && RulesEqual(a.Rules, a.Rules)
+	return ok && a.AtToken == b.AtToken && TokensEqual(a.Prelude, b.Prelude, check) && RulesEqual(a.Rules, b.Rules, check)
 }
 
 func (r *RKnownAt) Hash() (uint32, bool) {
@@ -390,9 +421,9 @@ type RUnknownAt struct {
 	Block   []Token
 }
 
-func (a *RUnknownAt) Equal(rule R) bool {
+func (a *RUnknownAt) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RUnknownAt)
-	return ok && a.AtToken == b.AtToken && TokensEqual(a.Prelude, b.Prelude) && TokensEqual(a.Block, a.Block)
+	return ok && a.AtToken == b.AtToken && TokensEqual(a.Prelude, b.Prelude, check) && TokensEqual(a.Block, b.Block, check)
 }
 
 func (r *RUnknownAt) Hash() (uint32, bool) {
@@ -406,18 +437,17 @@ func (r *RUnknownAt) Hash() (uint32, bool) {
 type RSelector struct {
 	Selectors []ComplexSelector
 	Rules     []Rule
-	HasAtNest bool
 }
 
-func (a *RSelector) Equal(rule R) bool {
+func (a *RSelector) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RSelector)
-	if ok && len(a.Selectors) == len(b.Selectors) && a.HasAtNest == b.HasAtNest {
-		for i, sel := range a.Selectors {
-			if !sel.Equal(b.Selectors[i]) {
+	if ok && len(a.Selectors) == len(b.Selectors) {
+		for i, ai := range a.Selectors {
+			if !ai.Equal(b.Selectors[i], check) {
 				return false
 			}
 		}
-		return RulesEqual(a.Rules, b.Rules)
+		return RulesEqual(a.Rules, b.Rules, check)
 	}
 
 	return false
@@ -438,7 +468,7 @@ func (r *RSelector) Hash() (uint32, bool) {
 			for _, sub := range sel.SubclassSelectors {
 				hash = helpers.HashCombine(hash, sub.Hash())
 			}
-			hash = helpers.HashCombineString(hash, sel.Combinator)
+			hash = helpers.HashCombine(hash, uint32(sel.Combinator))
 		}
 	}
 	hash = HashRules(hash, r.Rules)
@@ -450,9 +480,9 @@ type RQualified struct {
 	Rules   []Rule
 }
 
-func (a *RQualified) Equal(rule R) bool {
+func (a *RQualified) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RQualified)
-	return ok && TokensEqual(a.Prelude, b.Prelude) && RulesEqual(a.Rules, b.Rules)
+	return ok && TokensEqual(a.Prelude, b.Prelude, check) && RulesEqual(a.Rules, b.Rules, check)
 }
 
 func (r *RQualified) Hash() (uint32, bool) {
@@ -470,14 +500,28 @@ type RDeclaration struct {
 	Important bool
 }
 
-func (a *RDeclaration) Equal(rule R) bool {
+func (a *RDeclaration) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RDeclaration)
-	return ok && a.KeyText == b.KeyText && TokensEqual(a.Value, b.Value) && a.Important == b.Important
+	return ok && a.KeyText == b.KeyText && TokensEqual(a.Value, b.Value, check) && a.Important == b.Important
 }
 
 func (r *RDeclaration) Hash() (uint32, bool) {
-	hash := uint32(7)
-	hash = helpers.HashCombine(hash, uint32(r.Key))
+	var hash uint32
+	if r.Key == DUnknown {
+		if r.Important {
+			hash = uint32(7)
+		} else {
+			hash = uint32(8)
+		}
+		hash = helpers.HashCombineString(hash, r.KeyText)
+	} else {
+		if r.Important {
+			hash = uint32(9)
+		} else {
+			hash = uint32(10)
+		}
+		hash = helpers.HashCombine(hash, uint32(r.Key))
+	}
 	hash = HashTokens(hash, r.Value)
 	return hash, true
 }
@@ -486,13 +530,13 @@ type RBadDeclaration struct {
 	Tokens []Token
 }
 
-func (a *RBadDeclaration) Equal(rule R) bool {
+func (a *RBadDeclaration) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RBadDeclaration)
-	return ok && TokensEqual(a.Tokens, b.Tokens)
+	return ok && TokensEqual(a.Tokens, b.Tokens, check)
 }
 
 func (r *RBadDeclaration) Hash() (uint32, bool) {
-	hash := uint32(8)
+	hash := uint32(11)
 	hash = HashTokens(hash, r.Tokens)
 	return hash, true
 }
@@ -501,13 +545,13 @@ type RComment struct {
 	Text string
 }
 
-func (a *RComment) Equal(rule R) bool {
+func (a *RComment) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	b, ok := rule.(*RComment)
 	return ok && a.Text == b.Text
 }
 
 func (r *RComment) Hash() (uint32, bool) {
-	hash := uint32(9)
+	hash := uint32(12)
 	hash = helpers.HashCombineString(hash, r.Text)
 	return hash, true
 }
@@ -517,7 +561,7 @@ type RAtLayer struct {
 	Rules []Rule
 }
 
-func (a *RAtLayer) Equal(rule R) bool {
+func (a *RAtLayer) Equal(rule R, check *CrossFileEqualityCheck) bool {
 	if b, ok := rule.(*RAtLayer); ok && len(a.Names) == len(b.Names) && len(a.Rules) == len(b.Rules) {
 		for i, ai := range a.Names {
 			bi := b.Names[i]
@@ -530,7 +574,7 @@ func (a *RAtLayer) Equal(rule R) bool {
 				}
 			}
 		}
-		if !RulesEqual(a.Rules, b.Rules) {
+		if !RulesEqual(a.Rules, b.Rules, check) {
 			return false
 		}
 	}
@@ -538,7 +582,7 @@ func (a *RAtLayer) Equal(rule R) bool {
 }
 
 func (r *RAtLayer) Hash() (uint32, bool) {
-	hash := uint32(10)
+	hash := uint32(13)
 	hash = helpers.HashCombine(hash, uint32(len(r.Names)))
 	for _, parts := range r.Names {
 		hash = helpers.HashCombine(hash, uint32(len(parts)))
@@ -554,14 +598,78 @@ type ComplexSelector struct {
 	Selectors []CompoundSelector
 }
 
-func (a ComplexSelector) Equal(b ComplexSelector) bool {
+func (s ComplexSelector) AppendToTokensWithoutLeadingCombinator(tokens []Token) []Token {
+	for i, sel := range s.Selectors {
+		if n := len(tokens); i > 0 && n > 0 {
+			tokens[n-1].Whitespace |= WhitespaceAfter
+		}
+		if i == 0 {
+			sel.Combinator = 0
+		}
+		tokens = sel.AppendToTokens(tokens)
+	}
+	return tokens
+}
+
+func (sel ComplexSelector) IsRelative() bool {
+	if sel.Selectors[0].Combinator == 0 {
+		for _, inner := range sel.Selectors {
+			if inner.HasNestingSelector {
+				return false
+			}
+			for _, ss := range inner.SubclassSelectors {
+				if class, ok := ss.(*SSPseudoClass); ok && tokensContainAmpersandRecursive(class.Args) {
+					return false
+				}
+			}
+		}
+	}
+	return true
+}
+
+func tokensContainAmpersandRecursive(tokens []Token) bool {
+	for _, t := range tokens {
+		if t.Kind == css_lexer.TDelimAmpersand {
+			return true
+		}
+		if children := t.Children; children != nil && tokensContainAmpersandRecursive(*children) {
+			return true
+		}
+	}
+	return false
+}
+
+func (sel ComplexSelector) UsesPseudoElement() bool {
+	for _, sel := range sel.Selectors {
+		for _, sub := range sel.SubclassSelectors {
+			if class, ok := sub.(*SSPseudoClass); ok {
+				if class.IsElement {
+					return true
+				}
+
+				// https://www.w3.org/TR/selectors-4/#single-colon-pseudos
+				// The four Level 2 pseudo-elements (::before, ::after, ::first-line,
+				// and ::first-letter) may, for legacy reasons, be represented using
+				// the <pseudo-class-selector> grammar, with only a single ":"
+				// character at their start.
+				switch class.Name {
+				case "before", "after", "first-line", "first-letter":
+					return true
+				}
+			}
+		}
+	}
+	return false
+}
+
+func (a ComplexSelector) Equal(b ComplexSelector, check *CrossFileEqualityCheck) bool {
 	if len(a.Selectors) != len(b.Selectors) {
 		return false
 	}
 
 	for i, ai := range a.Selectors {
 		bi := b.Selectors[i]
-		if ai.NestingSelector != bi.NestingSelector || ai.Combinator != bi.Combinator {
+		if ai.HasNestingSelector != bi.HasNestingSelector || ai.Combinator != bi.Combinator {
 			return false
 		}
 
@@ -575,7 +683,7 @@ func (a ComplexSelector) Equal(b ComplexSelector) bool {
 			return false
 		}
 		for j, aj := range ai.SubclassSelectors {
-			if !aj.Equal(bi.SubclassSelectors[j]) {
+			if !aj.Equal(bi.SubclassSelectors[j], check) {
 				return false
 			}
 		}
@@ -584,19 +692,46 @@ func (a ComplexSelector) Equal(b ComplexSelector) bool {
 	return true
 }
 
-type NestingSelector uint8
-
-const (
-	NestingSelectorNone                NestingSelector = iota
-	NestingSelectorPrefix                              // "&a {}"
-	NestingSelectorPresentButNotPrefix                 // "a& {}"
-)
-
 type CompoundSelector struct {
-	Combinator        string // Optional, may be ""
-	TypeSelector      *NamespacedName
-	SubclassSelectors []SS
-	NestingSelector   NestingSelector // "&"
+	TypeSelector       *NamespacedName
+	SubclassSelectors  []SS
+	Combinator         uint8 // Optional, may be 0
+	HasNestingSelector bool  // "&"
+}
+
+func (sel CompoundSelector) IsSingleAmpersand() bool {
+	return sel.HasNestingSelector && sel.Combinator == 0 && sel.TypeSelector == nil && len(sel.SubclassSelectors) == 0
+}
+
+func (sel CompoundSelector) AppendToTokens(tokens []Token) []Token {
+	if sel.Combinator != 0 {
+		switch sel.Combinator {
+		case '>':
+			tokens = append(tokens, Token{Kind: css_lexer.TDelimGreaterThan, Text: ">", Whitespace: WhitespaceAfter})
+		case '+':
+			tokens = append(tokens, Token{Kind: css_lexer.TDelimPlus, Text: "+", Whitespace: WhitespaceAfter})
+		case '~':
+			tokens = append(tokens, Token{Kind: css_lexer.TDelimTilde, Text: "~", Whitespace: WhitespaceAfter})
+		default:
+			panic("Internal error")
+		}
+	}
+
+	if sel.TypeSelector != nil {
+		tokens = sel.TypeSelector.AppendToTokens(tokens)
+	}
+
+	// Put this after the type selector in case it's substituted for ":is()"
+	// ".foo { > &a, > &b {} }" => ".foo > :is(b.foo, c.foo) {}" (we don't want to get ".foo > :is(.foob, .fooc) {}" instead)
+	if sel.HasNestingSelector {
+		tokens = append(tokens, Token{Kind: css_lexer.TDelimAmpersand, Text: "&"})
+	}
+
+	for _, ss := range sel.SubclassSelectors {
+		tokens = ss.AppendToTokens(tokens)
+	}
+
+	return tokens
 }
 
 type NameToken struct {
@@ -612,21 +747,32 @@ type NamespacedName struct {
 	Name NameToken
 }
 
+func (n NamespacedName) AppendToTokens(tokens []Token) []Token {
+	if n.NamespacePrefix != nil {
+		tokens = append(tokens,
+			Token{Kind: n.NamespacePrefix.Kind, Text: n.NamespacePrefix.Text},
+			Token{Kind: css_lexer.TDelimBar, Text: "|"},
+		)
+	}
+	return append(tokens, Token{Kind: n.Name.Kind, Text: n.Name.Text})
+}
+
 func (a NamespacedName) Equal(b NamespacedName) bool {
 	return a.Name == b.Name && (a.NamespacePrefix == nil) == (b.NamespacePrefix == nil) &&
 		(a.NamespacePrefix == nil || b.NamespacePrefix == nil || *a.NamespacePrefix == *b.NamespacePrefix)
 }
 
 type SS interface {
-	Equal(ss SS) bool
+	Equal(ss SS, check *CrossFileEqualityCheck) bool
 	Hash() uint32
+	AppendToTokens(tokens []Token) []Token
 }
 
 type SSHash struct {
 	Name string
 }
 
-func (a *SSHash) Equal(ss SS) bool {
+func (a *SSHash) Equal(ss SS, check *CrossFileEqualityCheck) bool {
 	b, ok := ss.(*SSHash)
 	return ok && a.Name == b.Name
 }
@@ -637,11 +783,15 @@ func (ss *SSHash) Hash() uint32 {
 	return hash
 }
 
+func (ss *SSHash) AppendToTokens(tokens []Token) []Token {
+	return append(tokens, Token{Kind: css_lexer.THash, Text: ss.Name})
+}
+
 type SSClass struct {
 	Name string
 }
 
-func (a *SSClass) Equal(ss SS) bool {
+func (a *SSClass) Equal(ss SS, check *CrossFileEqualityCheck) bool {
 	b, ok := ss.(*SSClass)
 	return ok && a.Name == b.Name
 }
@@ -652,6 +802,13 @@ func (ss *SSClass) Hash() uint32 {
 	return hash
 }
 
+func (ss *SSClass) AppendToTokens(tokens []Token) []Token {
+	return append(tokens,
+		Token{Kind: css_lexer.TDelimDot, Text: "."},
+		Token{Kind: css_lexer.TIdent, Text: ss.Name},
+	)
+}
+
 type SSAttribute struct {
 	MatcherOp       string // Either "" or one of: "=" "~=" "|=" "^=" "$=" "*="
 	MatcherValue    string
@@ -659,7 +816,7 @@ type SSAttribute struct {
 	MatcherModifier byte // Either 0 or one of: 'i' 'I' 's' 'S'
 }
 
-func (a *SSAttribute) Equal(ss SS) bool {
+func (a *SSAttribute) Equal(ss SS, check *CrossFileEqualityCheck) bool {
 	b, ok := ss.(*SSAttribute)
 	return ok && a.NamespacedName.Equal(b.NamespacedName) && a.MatcherOp == b.MatcherOp &&
 		a.MatcherValue == b.MatcherValue && a.MatcherModifier == b.MatcherModifier
@@ -673,15 +830,64 @@ func (ss *SSAttribute) Hash() uint32 {
 	return hash
 }
 
+func (ss *SSAttribute) AppendToTokens(tokens []Token) []Token {
+	var children []Token
+	children = ss.NamespacedName.AppendToTokens(children)
+
+	if ss.MatcherOp != "" {
+		switch ss.MatcherOp {
+		case "=":
+			children = append(children, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		case "~=":
+			children = append(children, Token{Kind: css_lexer.TDelimTilde, Text: "~"}, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		case "|=":
+			children = append(children, Token{Kind: css_lexer.TDelimBar, Text: "|"}, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		case "^=":
+			children = append(children, Token{Kind: css_lexer.TDelimCaret, Text: "^"}, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		case "$=":
+			children = append(children, Token{Kind: css_lexer.TDelimDollar, Text: "$"}, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		case "*=":
+			children = append(children, Token{Kind: css_lexer.TDelimAsterisk, Text: "*"}, Token{Kind: css_lexer.TDelimEquals, Text: "="})
+		default:
+			panic("Internal error")
+		}
+		printAsIdent := false
+
+		// Print the value as an identifier if it's possible
+		if css_lexer.WouldStartIdentifierWithoutEscapes(ss.MatcherValue) {
+			printAsIdent = true
+			for _, c := range ss.MatcherValue {
+				if !css_lexer.IsNameContinue(c) {
+					printAsIdent = false
+					break
+				}
+			}
+		}
+
+		if printAsIdent {
+			children = append(children, Token{Kind: css_lexer.TIdent, Text: ss.MatcherValue})
+		} else {
+			children = append(children, Token{Kind: css_lexer.TString, Text: ss.MatcherValue})
+		}
+	}
+
+	if ss.MatcherModifier != 0 {
+		children = append(children, Token{Kind: css_lexer.TIdent, Text: string(rune(ss.MatcherModifier)), Whitespace: WhitespaceBefore})
+	}
+
+	tokens = append(tokens, Token{Kind: css_lexer.TOpenBracket, Text: "[", Children: &children})
+	return tokens
+}
+
 type SSPseudoClass struct {
 	Name      string
 	Args      []Token
 	IsElement bool // If true, this is prefixed by "::" instead of ":"
 }
 
-func (a *SSPseudoClass) Equal(ss SS) bool {
+func (a *SSPseudoClass) Equal(ss SS, check *CrossFileEqualityCheck) bool {
 	b, ok := ss.(*SSPseudoClass)
-	return ok && a.Name == b.Name && TokensEqual(a.Args, b.Args) && a.IsElement == b.IsElement
+	return ok && a.Name == b.Name && TokensEqual(a.Args, b.Args, check) && a.IsElement == b.IsElement
 }
 
 func (ss *SSPseudoClass) Hash() uint32 {
@@ -690,3 +896,21 @@ func (ss *SSPseudoClass) Hash() uint32 {
 	hash = HashTokens(hash, ss.Args)
 	return hash
 }
+
+func (ss *SSPseudoClass) AppendToTokens(tokens []Token) []Token {
+	if ss.IsElement {
+		tokens = append(tokens, Token{Kind: css_lexer.TColon, Text: ":"})
+	}
+
+	if ss.Args != nil {
+		return append(tokens,
+			Token{Kind: css_lexer.TColon, Text: ":"},
+			Token{Kind: css_lexer.TFunction, Text: ss.Name, Children: &ss.Args},
+		)
+	}
+
+	return append(tokens,
+		Token{Kind: css_lexer.TColon, Text: ":"},
+		Token{Kind: css_lexer.TIdent, Text: ss.Name},
+	)
+}
