@@ -15,18 +15,19 @@ import (
 // support for parsing https://drafts.csswg.org/css-nesting-1/.
 
 type parser struct {
-	log               logger.Log
-	source            logger.Source
-	tokens            []css_lexer.Token
-	legalComments     []css_lexer.Comment
-	stack             []css_lexer.T
-	importRecords     []ast.ImportRecord
-	tracker           logger.LineColumnTracker
-	index             int
-	end               int
-	legalCommentIndex int
-	prevError         logger.Loc
-	options           Options
+	log                logger.Log
+	source             logger.Source
+	tokens             []css_lexer.Token
+	legalComments      []css_lexer.Comment
+	stack              []css_lexer.T
+	importRecords      []ast.ImportRecord
+	tracker            logger.LineColumnTracker
+	index              int
+	end                int
+	legalCommentIndex  int
+	prevError          logger.Loc
+	options            Options
+	shouldLowerNesting bool
 }
 
 type Options struct {
@@ -138,7 +139,8 @@ func (p *parser) expectWithMatchingLoc(kind css_lexer.T, matchingLoc logger.Loc)
 		// Have a nice error message for forgetting a trailing semicolon or colon
 		text = fmt.Sprintf("Expected %s", expected)
 		t = p.at(p.index - 1)
-	} else if (kind == css_lexer.TCloseBrace || kind == css_lexer.TCloseBracket || kind == css_lexer.TCloseParen) && matchingLoc.Start != -1 {
+	} else if (kind == css_lexer.TCloseBrace || kind == css_lexer.TCloseBracket || kind == css_lexer.TCloseParen) &&
+		matchingLoc.Start != -1 && int(matchingLoc.Start)+1 <= len(p.source.Contents) {
 		// Have a nice error message for forgetting a closing brace/bracket/parenthesis
 		c := p.source.Contents[matchingLoc.Start : matchingLoc.Start+1]
 		text = fmt.Sprintf("Expected %s to go with %q", expected, c)
@@ -148,7 +150,7 @@ func (p *parser) expectWithMatchingLoc(kind css_lexer.T, matchingLoc logger.Loc)
 		case css_lexer.TEndOfFile, css_lexer.TWhitespace:
 			text = fmt.Sprintf("Expected %s but found %s", expected, t.Kind.String())
 			t.Range.Len = 0
-		case css_lexer.TBadURL, css_lexer.TBadString:
+		case css_lexer.TBadURL, css_lexer.TUnterminatedString:
 			text = fmt.Sprintf("Expected %s but found %s", expected, t.Kind.String())
 		default:
 			text = fmt.Sprintf("Expected %s but found %q", expected, p.raw())
@@ -171,7 +173,7 @@ func (p *parser) unexpected() {
 		case css_lexer.TEndOfFile, css_lexer.TWhitespace:
 			text = fmt.Sprintf("Unexpected %s", t.Kind.String())
 			t.Range.Len = 0
-		case css_lexer.TBadURL, css_lexer.TBadString:
+		case css_lexer.TBadURL, css_lexer.TUnterminatedString:
 			text = fmt.Sprintf("Unexpected %s", t.Kind.String())
 		default:
 			text = fmt.Sprintf("Unexpected %q", p.raw())
@@ -198,6 +200,10 @@ func (p *parser) parseListOfRules(context ruleContext) []css_ast.Rule {
 
 loop:
 	for {
+		if context.isTopLevel {
+			p.shouldLowerNesting = false
+		}
+
 		// If there are any legal comments immediately before the current token,
 		// turn them all into comment rules and append them to the current rule list
 		for p.legalCommentIndex < len(p.legalComments) {
@@ -265,7 +271,12 @@ loop:
 				}
 			}
 
-			rules = append(rules, rule)
+			// Lower CSS nesting if it's not supported (but only at the top level)
+			if context.isTopLevel && p.shouldLowerNesting {
+				rules = lowerNestingInRule(rule, rules)
+			} else {
+				rules = append(rules, rule)
+			}
 			continue
 
 		case css_lexer.TCDO, css_lexer.TCDC:
@@ -281,21 +292,35 @@ loop:
 			atRuleContext.importValidity = atRuleInvalidAfter
 		}
 
+		var rule css_ast.Rule
 		if context.parseSelectors {
-			rules = append(rules, p.parseSelectorRuleFrom(p.index, parseSelectorOpts{isTopLevel: context.isTopLevel}))
+			rule = p.parseSelectorRuleFrom(p.index, context.isTopLevel, parseSelectorOpts{})
 		} else {
-			rules = append(rules, p.parseQualifiedRuleFrom(p.index, parseQualifiedRuleOpts{isTopLevel: context.isTopLevel}))
+			rule = p.parseQualifiedRuleFrom(p.index, parseQualifiedRuleOpts{isTopLevel: context.isTopLevel})
+		}
+
+		// Lower CSS nesting if it's not supported (but only at the top level)
+		if context.isTopLevel && p.shouldLowerNesting {
+			rules = lowerNestingInRule(rule, rules)
+		} else {
+			rules = append(rules, rule)
 		}
 	}
 
 	if p.options.MinifySyntax {
-		rules = mangleRules(rules, context.isTopLevel)
+		rules = p.mangleRules(rules, context.isTopLevel)
 	}
 	return rules
 }
 
-func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
+type listOfDeclarationsOpts struct {
+	canInlineNoOpNesting bool
+}
+
+func (p *parser) parseListOfDeclarations(opts listOfDeclarationsOpts) (list []css_ast.Rule) {
 	list = []css_ast.Rule{}
+	foundNesting := false
+
 	for {
 		switch p.current().Kind {
 		case css_lexer.TWhitespace, css_lexer.TSemicolon:
@@ -304,19 +329,54 @@ func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
 		case css_lexer.TEndOfFile, css_lexer.TCloseBrace:
 			list = p.processDeclarations(list)
 			if p.options.MinifySyntax {
-				list = mangleRules(list, false /* isTopLevel */)
+				list = p.mangleRules(list, false /* isTopLevel */)
+
+				// Pull out all unnecessarily-nested declarations and stick them at the end
+				if opts.canInlineNoOpNesting {
+					// "a { & { x: y } }" => "a { x: y }"
+					// "a { & { b: c } d: e }" => "a { d: e; b: c }"
+					if foundNesting {
+						var inlineDecls []css_ast.Rule
+						n := 0
+						for _, rule := range list {
+							if rule, ok := rule.Data.(*css_ast.RSelector); ok && len(rule.Selectors) == 1 {
+								if sel := rule.Selectors[0]; len(sel.Selectors) == 1 && sel.Selectors[0].IsSingleAmpersand() {
+									inlineDecls = append(inlineDecls, rule.Rules...)
+									continue
+								}
+							}
+							list[n] = rule
+							n++
+						}
+						list = append(list[:n], inlineDecls...)
+					}
+				} else {
+					// "a, b::before { & { x: y } }" => "a, b::before { & { x: y } }"
+				}
 			}
 			return
 
 		case css_lexer.TAtKeyword:
+			p.reportUseOfNesting(p.current().Range, false)
 			list = append(list, p.parseAtRule(atRuleContext{
-				isDeclarationList: true,
-				allowNesting:      true,
+				isDeclarationList:    true,
+				canInlineNoOpNesting: opts.canInlineNoOpNesting,
 			}))
 
-		case css_lexer.TDelimAmpersand:
-			// Reference: https://drafts.csswg.org/css-nesting-1/
-			list = append(list, p.parseSelectorRuleFrom(p.index, parseSelectorOpts{allowNesting: true}))
+		// Reference: https://drafts.csswg.org/css-nesting-1/
+		case css_lexer.TDelimAmpersand,
+			css_lexer.TDelimDot,
+			css_lexer.THash,
+			css_lexer.TColon,
+			css_lexer.TOpenBracket,
+			css_lexer.TDelimAsterisk,
+			css_lexer.TDelimBar,
+			css_lexer.TDelimPlus,
+			css_lexer.TDelimGreaterThan,
+			css_lexer.TDelimTilde:
+			p.reportUseOfNesting(p.current().Range, false)
+			list = append(list, p.parseSelectorRuleFrom(p.index, false, parseSelectorOpts{isDeclarationContext: true}))
+			foundNesting = true
 
 		default:
 			list = append(list, p.parseDeclaration())
@@ -324,7 +384,7 @@ func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
 	}
 }
 
-func mangleRules(rules []css_ast.Rule, isTopLevel bool) []css_ast.Rule {
+func (p *parser) mangleRules(rules []css_ast.Rule, isTopLevel bool) []css_ast.Rule {
 	type hashEntry struct {
 		indices []uint32
 	}
@@ -364,7 +424,7 @@ func mangleRules(rules []css_ast.Rule, isTopLevel bool) []css_ast.Rule {
 			}
 
 		case *css_ast.RKnownAt:
-			if len(r.Rules) == 0 {
+			if len(r.Rules) == 0 && atKnownRuleCanBeRemovedIfEmpty[r.AtToken] {
 				continue
 			}
 
@@ -377,12 +437,12 @@ func mangleRules(rules []css_ast.Rule, isTopLevel bool) []css_ast.Rule {
 			// "a { color: red; } b { color: red; }" => "a, b { color: red; }"
 			if prevNonComment != nil {
 				if r, ok := rule.Data.(*css_ast.RSelector); ok {
-					if prev, ok := prevNonComment.(*css_ast.RSelector); ok && css_ast.RulesEqual(r.Rules, prev.Rules) &&
+					if prev, ok := prevNonComment.(*css_ast.RSelector); ok && css_ast.RulesEqual(r.Rules, prev.Rules, nil) &&
 						isSafeSelectors(r.Selectors) && isSafeSelectors(prev.Selectors) {
 					nextSelector:
 						for _, sel := range r.Selectors {
 							for _, prevSel := range prev.Selectors {
-								if sel.Equal(prevSel) {
+								if sel.Equal(prevSel, nil) {
 									// Don't add duplicate selectors more than once
 									continue nextSelector
 								}
@@ -410,25 +470,39 @@ func mangleRules(rules []css_ast.Rule, isTopLevel bool) []css_ast.Rule {
 	// Mangle non-top-level rules using a back-to-front pass. Top-level rules
 	// will be mangled by the linker instead for cross-file rule mangling.
 	if !isTopLevel {
-		rules = MakeDuplicateRuleMangler().RemoveDuplicateRulesInPlace(rules)
+		remover := MakeDuplicateRuleMangler()
+		rules = remover.RemoveDuplicateRulesInPlace(rules, p.importRecords)
 	}
 
 	return rules
 }
 
+type ruleEntry struct {
+	data        css_ast.R
+	callCounter uint32
+}
+
 type hashEntry struct {
-	rules []css_ast.R
+	rules []ruleEntry
 }
 
 type DuplicateRuleRemover struct {
 	entries map[uint32]hashEntry
+	calls   [][]ast.ImportRecord
+	check   css_ast.CrossFileEqualityCheck
 }
 
 func MakeDuplicateRuleMangler() DuplicateRuleRemover {
 	return DuplicateRuleRemover{entries: make(map[uint32]hashEntry)}
 }
 
-func (remover DuplicateRuleRemover) RemoveDuplicateRulesInPlace(rules []css_ast.Rule) []css_ast.Rule {
+func (remover *DuplicateRuleRemover) RemoveDuplicateRulesInPlace(rules []css_ast.Rule, importRecords []ast.ImportRecord) []css_ast.Rule {
+	// The caller may call this function multiple times, each with a different
+	// set of import records. Remember each set of import records for equality
+	// checks later.
+	callCounter := uint32(len(remover.calls))
+	remover.calls = append(remover.calls, importRecords)
+
 	// Remove duplicate rules, scanning from the back so we keep the last
 	// duplicate. Note that the linker calls this, so we do not want to do
 	// anything that modifies the rules themselves. One reason is that ASTs
@@ -444,12 +518,27 @@ skipRule:
 		// For duplicate rules, omit all but the last copy
 		if hash, ok := rule.Data.Hash(); ok {
 			entry := remover.entries[hash]
-			for _, data := range entry.rules {
-				if rule.Data.Equal(data) {
+			for _, current := range entry.rules {
+				var check *css_ast.CrossFileEqualityCheck
+
+				// If this rule was from another file, then pass along both arrays
+				// of import records so that the equality check for "url()" tokens
+				// can use them to check for equality.
+				if current.callCounter != callCounter {
+					// Reuse the same memory allocation
+					check = &remover.check
+					check.ImportRecordsA = importRecords
+					check.ImportRecordsB = remover.calls[current.callCounter]
+				}
+
+				if rule.Data.Equal(current.data, check) {
 					continue skipRule
 				}
 			}
-			entry.rules = append(entry.rules, rule.Data)
+			entry.rules = append(entry.rules, ruleEntry{
+				data:        rule.Data,
+				callCounter: callCounter,
+			})
 			remover.entries[hash] = entry
 		}
 
@@ -566,12 +655,12 @@ var nonDeprecatedElementsSupportedByIE7 = map[string]bool{
 func isSafeSelectors(complexSelectors []css_ast.ComplexSelector) bool {
 	for _, complex := range complexSelectors {
 		for _, compound := range complex.Selectors {
-			if compound.NestingSelector != css_ast.NestingSelectorNone {
+			if compound.HasNestingSelector {
 				// Bail because this is an extension: https://drafts.csswg.org/css-nesting-1/
 				return false
 			}
 
-			if compound.Combinator != "" {
+			if compound.Combinator != 0 {
 				// "Before Internet Explorer 10, the combinator only works in standards mode"
 				// Reference: https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Selectors
 				return false
@@ -664,6 +753,9 @@ const (
 )
 
 var specialAtRules = map[string]atRuleKind{
+	"media":    atRuleInheritContext,
+	"supports": atRuleInheritContext,
+
 	"font-face": atRuleDeclarations,
 	"page":      atRuleDeclarations,
 
@@ -715,12 +807,8 @@ var specialAtRules = map[string]atRuleKind{
 	//
 	"layer": atRuleQualifiedOrEmpty,
 
-	"media":    atRuleInheritContext,
-	"scope":    atRuleInheritContext,
-	"supports": atRuleInheritContext,
-
-	// Reference: https://drafts.csswg.org/css-nesting-1/
-	"nest": atRuleDeclarations,
+	// Reference: https://drafts.csswg.org/css-cascade-6/#scoped-styles
+	"scope": atRuleInheritContext,
 
 	// Reference: https://drafts.csswg.org/css-fonts-4/#font-palette-values
 	"font-palette-values": atRuleDeclarations,
@@ -745,6 +833,40 @@ var specialAtRules = map[string]atRuleKind{
 	"container": atRuleInheritContext,
 }
 
+var atKnownRuleCanBeRemovedIfEmpty = map[string]bool{
+	"media":     true,
+	"supports":  true,
+	"font-face": true,
+	"page":      true,
+
+	// https://www.w3.org/TR/css-page-3/#syntax-page-selector
+	"bottom-center":       true,
+	"bottom-left-corner":  true,
+	"bottom-left":         true,
+	"bottom-right-corner": true,
+	"bottom-right":        true,
+	"left-bottom":         true,
+	"left-middle":         true,
+	"left-top":            true,
+	"right-bottom":        true,
+	"right-middle":        true,
+	"right-top":           true,
+	"top-center":          true,
+	"top-left-corner":     true,
+	"top-left":            true,
+	"top-right-corner":    true,
+	"top-right":           true,
+
+	// https://drafts.csswg.org/css-cascade-6/#scoped-styles
+	"scope": true,
+
+	// https://drafts.csswg.org/css-fonts-4/#font-palette-values
+	"font-palette-values": true,
+
+	// https://drafts.csswg.org/css-contain-3/#container-rule
+	"container": true,
+}
+
 type atRuleValidity uint8
 
 const (
@@ -754,12 +876,12 @@ const (
 )
 
 type atRuleContext struct {
-	afterLoc          logger.Loc
-	charsetValidity   atRuleValidity
-	importValidity    atRuleValidity
-	isDeclarationList bool
-	allowNesting      bool
-	isTopLevel        bool
+	afterLoc             logger.Loc
+	charsetValidity      atRuleValidity
+	importValidity       atRuleValidity
+	canInlineNoOpNesting bool
+	isDeclarationList    bool
+	isTopLevel           bool
 }
 
 func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
@@ -913,7 +1035,7 @@ abortRuleParser:
 						case css_lexer.TOpenBrace:
 							blockMatchingLoc := p.current().Range.Loc
 							p.advance()
-							rules := p.parseListOfDeclarations()
+							rules := p.parseListOfDeclarations(listOfDeclarationsOpts{})
 							p.expectWithMatchingLoc(css_lexer.TCloseBrace, blockMatchingLoc)
 
 							// "@keyframes { from {} to { color: red } }" => "@keyframes { to { color: red } }"
@@ -976,18 +1098,6 @@ abortRuleParser:
 			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
 		}
 
-	case "nest":
-		// Reference: https://drafts.csswg.org/css-nesting-1/
-		p.eat(css_lexer.TWhitespace)
-		if kind := p.current().Kind; kind != css_lexer.TSemicolon && kind != css_lexer.TOpenBrace &&
-			kind != css_lexer.TCloseBrace && kind != css_lexer.TEndOfFile {
-			return p.parseSelectorRuleFrom(preludeStart-1, parseSelectorOpts{
-				atNestRange:  atRange,
-				allowNesting: context.allowNesting,
-				isTopLevel:   context.isTopLevel,
-			})
-		}
-
 	case "layer":
 		// Reference: https://developer.mozilla.org/en-US/docs/Web/CSS/@layer
 
@@ -1025,9 +1135,16 @@ abortRuleParser:
 		// Read the optional block
 		matchingLoc := p.current().Range.Loc
 		if len(names) <= 1 && p.eat(css_lexer.TOpenBrace) {
-			rules := p.parseListOfRules(ruleContext{
-				parseSelectors: true,
-			})
+			var rules []css_ast.Rule
+			if context.isDeclarationList {
+				rules = p.parseListOfDeclarations(listOfDeclarationsOpts{
+					canInlineNoOpNesting: context.canInlineNoOpNesting,
+				})
+			} else {
+				rules = p.parseListOfRules(ruleContext{
+					parseSelectors: true,
+				})
+			}
 			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names, Rules: rules}}
 		}
@@ -1038,9 +1155,21 @@ abortRuleParser:
 		}
 
 		// Otherwise there's some kind of syntax error
-		if kind := p.current().Kind; kind == css_lexer.TOpenBrace || kind == css_lexer.TCloseBrace || kind == css_lexer.TEndOfFile {
+		switch p.current().Kind {
+		case css_lexer.TEndOfFile:
 			p.expect(css_lexer.TSemicolon)
-		} else {
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names}}
+
+		case css_lexer.TCloseBrace:
+			p.expect(css_lexer.TSemicolon)
+			if !context.isTopLevel {
+				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtLayer{Names: names}}
+			}
+
+		case css_lexer.TOpenBrace:
+			p.expect(css_lexer.TSemicolon)
+
+		default:
 			p.unexpected()
 		}
 
@@ -1112,7 +1241,7 @@ prelude:
 		// Parse known rules whose blocks always consist of declarations
 		matchingLoc := p.current().Range.Loc
 		p.expect(css_lexer.TOpenBrace)
-		rules := p.parseListOfDeclarations()
+		rules := p.parseListOfDeclarations(listOfDeclarationsOpts{})
 		p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
@@ -1122,7 +1251,9 @@ prelude:
 		p.expect(css_lexer.TOpenBrace)
 		var rules []css_ast.Rule
 		if context.isDeclarationList {
-			rules = p.parseListOfDeclarations()
+			rules = p.parseListOfDeclarations(listOfDeclarationsOpts{
+				canInlineNoOpNesting: context.canInlineNoOpNesting,
+			})
 		} else {
 			rules = p.parseListOfRules(ruleContext{
 				parseSelectors: true,
@@ -1166,6 +1297,19 @@ func (p *parser) expectValidLayerNameIdent() (string, bool) {
 	return text, true
 }
 
+func (p *parser) reportUseOfNesting(r logger.Range, didWarnAlready bool) {
+	if p.options.UnsupportedCSSFeatures.Has(compat.Nesting) {
+		p.shouldLowerNesting = true
+		if p.options.UnsupportedCSSFeatures.Has(compat.IsPseudoClass) && !didWarnAlready {
+			text := "CSS nesting syntax is not supported in the configured target environment"
+			if p.options.OriginalTargetEnv != "" {
+				text = fmt.Sprintf("%s (%s)", text, p.options.OriginalTargetEnv)
+			}
+			p.log.AddID(logger.MsgID_CSS_UnsupportedCSSNesting, logger.Warning, &p.tracker, r, text)
+		}
+	}
+}
+
 func (p *parser) convertTokens(tokens []css_lexer.Token) []css_ast.Token {
 	result, _ := p.convertTokensHelper(tokens, css_lexer.TEndOfFile, convertTokensOpts{})
 	return result
@@ -1178,7 +1322,7 @@ type convertTokensOpts struct {
 }
 
 func (p *parser) convertTokensHelper(tokens []css_lexer.Token, close css_lexer.T, opts convertTokensOpts) ([]css_ast.Token, []css_lexer.Token) {
-	var result []css_ast.Token
+	result := []css_ast.Token{}
 	var nextWhitespace css_ast.WhitespaceFlags
 
 	// Enable verbatim whitespace mode when the first two non-whitespace tokens
@@ -1513,32 +1657,30 @@ func mangleNumber(t string) (string, bool) {
 	return t, t != original
 }
 
-func (p *parser) parseSelectorRuleFrom(preludeStart int, opts parseSelectorOpts) css_ast.Rule {
+func (p *parser) parseSelectorRuleFrom(preludeStart int, isTopLevel bool, opts parseSelectorOpts) css_ast.Rule {
 	// Try parsing the prelude as a selector list
 	if list, ok := p.parseSelectorList(opts); ok {
-		selector := css_ast.RSelector{
-			Selectors: list,
-			HasAtNest: opts.atNestRange.Len != 0,
+		canInlineNoOpNesting := true
+		for _, sel := range list {
+			// We cannot transform the CSS "a, b::before { & { color: red } }" into
+			// "a, b::before { color: red }" because it's basically equivalent to
+			// ":is(a, b::before) { color: red }" which only applies to "a", not to
+			// "b::before" because pseudo-elements are not valid within :is():
+			// https://www.w3.org/TR/selectors-4/#matches-pseudo. This restriction
+			// may be relaxed in the future, but this restriction hash shipped so
+			// we're stuck with it: https://github.com/w3c/csswg-drafts/issues/7433.
+			if sel.UsesPseudoElement() {
+				canInlineNoOpNesting = false
+				break
+			}
 		}
+		selector := css_ast.RSelector{Selectors: list}
 		matchingLoc := p.current().Range.Loc
 		if p.expect(css_lexer.TOpenBrace) {
-			selector.Rules = p.parseListOfDeclarations()
+			selector.Rules = p.parseListOfDeclarations(listOfDeclarationsOpts{
+				canInlineNoOpNesting: canInlineNoOpNesting,
+			})
 			p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
-
-			// Minify "@nest" when possible
-			if p.options.MinifySyntax && selector.HasAtNest {
-				allHaveNestPrefix := true
-				for _, complex := range selector.Selectors {
-					if len(complex.Selectors) == 0 || complex.Selectors[0].NestingSelector != css_ast.NestingSelectorPrefix {
-						allHaveNestPrefix = false
-						break
-					}
-				}
-				if allHaveNestPrefix {
-					selector.HasAtNest = false
-				}
-			}
-
 			return css_ast.Rule{Loc: p.tokens[preludeStart].Range.Loc, Data: &selector}
 		}
 	}
@@ -1546,7 +1688,7 @@ func (p *parser) parseSelectorRuleFrom(preludeStart int, opts parseSelectorOpts)
 	// Otherwise, parse a generic qualified rule
 	return p.parseQualifiedRuleFrom(preludeStart, parseQualifiedRuleOpts{
 		isAlreadyInvalid: true,
-		isTopLevel:       opts.isTopLevel,
+		isTopLevel:       isTopLevel,
 	})
 }
 
@@ -1581,7 +1723,7 @@ loop:
 
 	matchingLoc := p.current().Range.Loc
 	if p.eat(css_lexer.TOpenBrace) {
-		qualified.Rules = p.parseListOfDeclarations()
+		qualified.Rules = p.parseListOfDeclarations(listOfDeclarationsOpts{})
 		p.expectWithMatchingLoc(css_lexer.TCloseBrace, matchingLoc)
 	} else if !opts.isAlreadyInvalid {
 		p.expect(css_lexer.TOpenBrace)
@@ -1593,23 +1735,27 @@ loop:
 func (p *parser) parseDeclaration() css_ast.Rule {
 	// Parse the key
 	keyStart := p.index
-	keyLoc := p.tokens[keyStart].Range.Loc
+	keyRange := p.tokens[keyStart].Range
+	keyIsIdent := p.expect(css_lexer.TIdent)
 	ok := false
-	if p.expect(css_lexer.TIdent) {
+	if keyIsIdent {
 		p.eat(css_lexer.TWhitespace)
-		if p.expect(css_lexer.TColon) {
-			ok = true
-		}
+		ok = p.eat(css_lexer.TColon)
 	}
 
 	// Parse the value
 	valueStart := p.index
+	foundOpenBrace := false
 stop:
 	for {
 		switch p.current().Kind {
 		case css_lexer.TEndOfFile, css_lexer.TSemicolon, css_lexer.TCloseBrace:
 			break stop
 
+		case css_lexer.TOpenBrace:
+			foundOpenBrace = true
+			p.parseComponentValue()
+
 		default:
 			p.parseComponentValue()
 		}
@@ -1617,7 +1763,37 @@ stop:
 
 	// Stop now if this is not a valid declaration
 	if !ok {
-		return css_ast.Rule{Loc: keyLoc, Data: &css_ast.RBadDeclaration{
+		if keyIsIdent {
+			if foundOpenBrace {
+				// If we encountered a "{", assume this is someone trying to make a nested style rule
+				if keyRange.Loc.Start > p.prevError.Start {
+					p.prevError.Start = keyRange.Loc.Start
+					key := p.tokens[keyStart].DecodedText(p.source.Contents)
+					data := p.tracker.MsgData(keyRange, fmt.Sprintf("A nested style rule cannot start with %q because it looks like the start of a declaration", key))
+					data.Location.Suggestion = fmt.Sprintf(":is(%s)", p.source.TextForRange(keyRange))
+					p.log.AddMsgID(logger.MsgID_CSS_CSSSyntaxError, logger.Msg{
+						Kind: logger.Warning,
+						Data: data,
+						Notes: []logger.MsgData{{
+							Text: "To start a nested style rule with an identifier, you need to wrap the " +
+								"identifier in \":is(...)\" to prevent the rule from being parsed as a declaration."}},
+					})
+				}
+			} else {
+				// Otherwise, show a generic error about a missing ":"
+				if end := keyRange.End(); end > p.prevError.Start {
+					p.prevError.Start = end
+					data := p.tracker.MsgData(logger.Range{Loc: logger.Loc{Start: end}}, "Expected \":\"")
+					data.Location.Suggestion = ":"
+					p.log.AddMsgID(logger.MsgID_CSS_CSSSyntaxError, logger.Msg{
+						Kind: logger.Warning,
+						Data: data,
+					})
+				}
+			}
+		}
+
+		return css_ast.Rule{Loc: keyRange.Loc, Data: &css_ast.RBadDeclaration{
 			Tokens: p.convertTokens(p.tokens[keyStart:p.index]),
 		}}
 	}
@@ -1672,7 +1848,7 @@ stop:
 		}
 	}
 
-	return css_ast.Rule{Loc: keyLoc, Data: &css_ast.RDeclaration{
+	return css_ast.Rule{Loc: keyRange.Loc, Data: &css_ast.RDeclaration{
 		Key:       key,
 		KeyText:   keyText,
 		KeyRange:  keyToken.Range,
@@ -1704,11 +1880,12 @@ func (p *parser) parseComponentValue() {
 }
 
 func (p *parser) parseBlock(open css_lexer.T, close css_lexer.T) {
-	matchingLoc := p.current().Range.Loc
+	current := p.current()
+	matchingStart := current.Range.End() - 1
 	if p.expect(open) {
 		for !p.eat(close) {
 			if p.peek(css_lexer.TEndOfFile) {
-				p.expectWithMatchingLoc(close, matchingLoc)
+				p.expectWithMatchingLoc(close, logger.Loc{Start: matchingStart})
 				return
 			}
 
