@@ -1,9 +1,6 @@
 package css_parser
 
 import (
-	"fmt"
-
-	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/css_ast"
 	"github.com/evanw/esbuild/internal/css_lexer"
 	"github.com/evanw/esbuild/internal/logger"
@@ -11,36 +8,56 @@ import (
 
 func (p *parser) parseSelectorList(opts parseSelectorOpts) (list []css_ast.ComplexSelector, ok bool) {
 	// Parse the first selector
-	firstRange := p.current().Range
-	sel, good, firstHasNestPrefix := p.parseComplexSelector(opts)
+	sel, good := p.parseComplexSelector(opts)
 	if !good {
 		return
 	}
 	list = append(list, sel)
 
 	// Parse the remaining selectors
+skip:
 	for {
 		p.eat(css_lexer.TWhitespace)
 		if !p.eat(css_lexer.TComma) {
 			break
 		}
 		p.eat(css_lexer.TWhitespace)
-		loc := p.current().Range.Loc
-		sel, good, hasNestPrefix := p.parseComplexSelector(opts)
+		sel, good := p.parseComplexSelector(opts)
 		if !good {
 			return
 		}
-		list = append(list, sel)
 
-		// Validate nest prefix consistency
-		if firstHasNestPrefix && !hasNestPrefix && opts.atNestRange.Len == 0 {
-			data := p.tracker.MsgData(logger.Range{Loc: loc}, "Every selector in a nested style rule must start with \"&\"")
-			data.Location.Suggestion = "&"
-			p.log.AddMsgID(logger.MsgID_CSS_InvalidAtNest, logger.Msg{
-				Kind:  logger.Warning,
-				Data:  data,
-				Notes: []logger.MsgData{p.tracker.MsgData(firstRange, "This is a nested style rule because of the \"&\" here:")},
-			})
+		// Omit duplicate selectors
+		if p.options.MinifySyntax {
+			for _, existing := range list {
+				if sel.Equal(existing, nil) {
+					continue skip
+				}
+			}
+		}
+
+		list = append(list, sel)
+	}
+
+	if p.options.MinifySyntax {
+		for i := 1; i < len(list); i++ {
+			if analyzeLeadingAmpersand(list[i], opts.isDeclarationContext) != cannotRemoveLeadingAmpersand {
+				list[i].Selectors = list[i].Selectors[1:]
+			}
+		}
+
+		switch analyzeLeadingAmpersand(list[0], opts.isDeclarationContext) {
+		case canAlwaysRemoveLeadingAmpersand:
+			list[0].Selectors = list[0].Selectors[1:]
+
+		case canRemoveLeadingAmpersandIfNotFirst:
+			for i := 1; i < len(list); i++ {
+				if sel := list[i].Selectors[0]; !sel.HasNestingSelector && (sel.Combinator != 0 || sel.TypeSelector == nil) {
+					list[0].Selectors = list[0].Selectors[1:]
+					list[0], list[i] = list[i], list[0]
+					break
+				}
+			}
 		}
 	}
 
@@ -48,21 +65,57 @@ func (p *parser) parseSelectorList(opts parseSelectorOpts) (list []css_ast.Compl
 	return
 }
 
-type parseSelectorOpts struct {
-	atNestRange  logger.Range
-	allowNesting bool
-	isTopLevel   bool
+type leadingAmpersand uint8
+
+const (
+	cannotRemoveLeadingAmpersand leadingAmpersand = iota
+	canAlwaysRemoveLeadingAmpersand
+	canRemoveLeadingAmpersandIfNotFirst
+)
+
+func analyzeLeadingAmpersand(sel css_ast.ComplexSelector, isDeclarationContext bool) leadingAmpersand {
+	if len(sel.Selectors) > 1 {
+		if first := sel.Selectors[0]; first.IsSingleAmpersand() {
+			if second := sel.Selectors[1]; second.Combinator == 0 && second.HasNestingSelector {
+				// ".foo { & &.bar {} }" => ".foo { & &.bar {} }"
+			} else if second.Combinator != 0 || second.TypeSelector == nil || !isDeclarationContext {
+				// "& + div {}" => "+ div {}"
+				// "& div {}" => "div {}"
+				// ".foo { & + div {} }" => ".foo { + div {} }"
+				// ".foo { & + &.bar {} }" => ".foo { + &.bar {} }"
+				// ".foo { & :hover {} }" => ".foo { :hover {} }"
+				return canAlwaysRemoveLeadingAmpersand
+			} else {
+				// ".foo { & div {} }"
+				// ".foo { .bar, & div {} }" => ".foo { .bar, div {} }"
+				return canRemoveLeadingAmpersandIfNotFirst
+			}
+		}
+	} else {
+		// "& {}" => "& {}"
+	}
+	return cannotRemoveLeadingAmpersand
 }
 
-func (p *parser) parseComplexSelector(opts parseSelectorOpts) (result css_ast.ComplexSelector, ok bool, hasNestPrefix bool) {
+type parseSelectorOpts struct {
+	isDeclarationContext bool
+}
+
+func (p *parser) parseComplexSelector(opts parseSelectorOpts) (result css_ast.ComplexSelector, ok bool) {
+	// This is an extension: https://drafts.csswg.org/css-nesting-1/
+	r := p.current().Range
+	combinator := p.parseCombinator()
+	if combinator != 0 {
+		p.reportUseOfNesting(r, opts.isDeclarationContext)
+		p.eat(css_lexer.TWhitespace)
+	}
+
 	// Parent
-	loc := p.current().Range.Loc
 	sel, good := p.parseCompoundSelector(opts)
 	if !good {
 		return
 	}
-	hasNestPrefix = sel.NestingSelector == css_ast.NestingSelectorPrefix
-	isNestContaining := sel.NestingSelector != css_ast.NestingSelectorNone
+	sel.Combinator = combinator
 	result.Selectors = append(result.Selectors, sel)
 
 	for {
@@ -73,7 +126,7 @@ func (p *parser) parseComplexSelector(opts parseSelectorOpts) (result css_ast.Co
 
 		// Optional combinator
 		combinator := p.parseCombinator()
-		if combinator != "" {
+		if combinator != 0 {
 			p.eat(css_lexer.TWhitespace)
 		}
 
@@ -84,16 +137,6 @@ func (p *parser) parseComplexSelector(opts parseSelectorOpts) (result css_ast.Co
 		}
 		sel.Combinator = combinator
 		result.Selectors = append(result.Selectors, sel)
-		if sel.NestingSelector != css_ast.NestingSelectorNone {
-			isNestContaining = true
-		}
-	}
-
-	// Validate nest selector consistency
-	if opts.atNestRange.Len != 0 && !isNestContaining {
-		p.log.AddIDWithNotes(logger.MsgID_CSS_InvalidAtNest, logger.Warning, &p.tracker, logger.Range{Loc: loc},
-			"Every selector in a nested style rule must contain \"&\"",
-			[]logger.MsgData{p.tracker.MsgData(opts.atNestRange, "This is a nested style rule because of the \"@nest\" here:")})
 	}
 
 	ok = true
@@ -107,24 +150,12 @@ func (p *parser) nameToken() css_ast.NameToken {
 	}
 }
 
-func (p *parser) maybeWarnAboutNesting(r logger.Range, opts parseSelectorOpts) {
-	if !opts.allowNesting {
-		p.log.AddID(logger.MsgID_CSS_InvalidAtNest, logger.Warning, &p.tracker, r, "CSS nesting syntax cannot be used outside of a style rule")
-	} else if p.options.UnsupportedCSSFeatures.Has(compat.Nesting) {
-		text := "CSS nesting syntax is not supported in the configured target environment"
-		if p.options.OriginalTargetEnv != "" {
-			text = fmt.Sprintf("%s (%s)", text, p.options.OriginalTargetEnv)
-		}
-		p.log.AddID(logger.MsgID_CSS_InvalidAtNest, logger.Warning, &p.tracker, r, text)
-	}
-}
-
 func (p *parser) parseCompoundSelector(opts parseSelectorOpts) (sel css_ast.CompoundSelector, ok bool) {
 	// This is an extension: https://drafts.csswg.org/css-nesting-1/
-	r := p.current().Range
-	if p.eat(css_lexer.TDelimAmpersand) {
-		sel.NestingSelector = css_ast.NestingSelectorPrefix
-		p.maybeWarnAboutNesting(r, opts)
+	if p.peek(css_lexer.TDelimAmpersand) {
+		p.reportUseOfNesting(p.current().Range, opts.isDeclarationContext)
+		sel.HasNestingSelector = true
+		p.advance()
 	}
 
 	// Parse the type selector
@@ -208,12 +239,9 @@ subclassSelectors:
 
 		case css_lexer.TDelimAmpersand:
 			// This is an extension: https://drafts.csswg.org/css-nesting-1/
-			r := p.current().Range
+			p.reportUseOfNesting(p.current().Range, sel.HasNestingSelector)
+			sel.HasNestingSelector = true
 			p.advance()
-			if sel.NestingSelector == css_ast.NestingSelectorNone {
-				sel.NestingSelector = css_ast.NestingSelectorPresentButNotPrefix
-				p.maybeWarnAboutNesting(r, opts)
-			}
 
 		default:
 			break subclassSelectors
@@ -221,7 +249,7 @@ subclassSelectors:
 	}
 
 	// The compound selector must be non-empty
-	if sel.NestingSelector == css_ast.NestingSelectorNone && sel.TypeSelector == nil && len(sel.SubclassSelectors) == 0 {
+	if !sel.HasNestingSelector && sel.TypeSelector == nil && len(sel.SubclassSelectors) == 0 {
 		p.unexpected()
 		return
 	}
@@ -386,21 +414,21 @@ loop:
 	return tokens
 }
 
-func (p *parser) parseCombinator() string {
+func (p *parser) parseCombinator() uint8 {
 	switch p.current().Kind {
 	case css_lexer.TDelimGreaterThan:
 		p.advance()
-		return ">"
+		return '>'
 
 	case css_lexer.TDelimPlus:
 		p.advance()
-		return "+"
+		return '+'
 
 	case css_lexer.TDelimTilde:
 		p.advance()
-		return "~"
+		return '~'
 
 	default:
-		return ""
+		return 0
 	}
 }
