@@ -211,24 +211,21 @@ func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicol
 			whitespace = canDiscardWhitespaceAfter
 		}
 		p.printIdent(r.AtToken, identNormal, whitespace)
-		if (!p.options.MinifyWhitespace && r.Block != nil) || len(r.Prelude) > 0 {
+		if (!p.options.MinifyWhitespace && len(r.Block) != 0) || len(r.Prelude) > 0 {
 			p.print(" ")
 		}
 		p.printTokens(r.Prelude, printTokensOpts{})
-		if !p.options.MinifyWhitespace && r.Block != nil && len(r.Prelude) > 0 {
+		if !p.options.MinifyWhitespace && len(r.Block) != 0 && len(r.Prelude) > 0 {
 			p.print(" ")
 		}
-		if r.Block == nil {
+		if len(r.Block) == 0 {
 			p.print(";")
 		} else {
 			p.printTokens(r.Block, printTokensOpts{})
 		}
 
 	case *css_ast.RSelector:
-		if r.HasAtNest {
-			p.print("@nest")
-		}
-		p.printComplexSelectors(r.Selectors, indent, r.HasAtNest)
+		p.printComplexSelectors(r.Selectors, indent)
 		if !p.options.MinifyWhitespace {
 			p.print(" ")
 		}
@@ -336,7 +333,7 @@ func (p *printer) printRuleBlock(rules []css_ast.Rule, indent int32) {
 	p.print("}")
 }
 
-func (p *printer) printComplexSelectors(selectors []css_ast.ComplexSelector, indent int32, hasAtNest bool) {
+func (p *printer) printComplexSelectors(selectors []css_ast.ComplexSelector, indent int32) {
 	for i, complex := range selectors {
 		if i > 0 {
 			if p.options.MinifyWhitespace {
@@ -348,31 +345,31 @@ func (p *printer) printComplexSelectors(selectors []css_ast.ComplexSelector, ind
 		}
 
 		for j, compound := range complex.Selectors {
-			p.printCompoundSelector(compound, (!hasAtNest || i != 0) && j == 0, j+1 == len(complex.Selectors))
+			p.printCompoundSelector(compound, j == 0, j+1 == len(complex.Selectors))
 		}
 	}
 }
 
 func (p *printer) printCompoundSelector(sel css_ast.CompoundSelector, isFirst bool, isLast bool) {
-	if !isFirst && sel.Combinator == "" {
+	if !isFirst && sel.Combinator == 0 {
 		// A space is required in between compound selectors if there is no
 		// combinator in the middle. It's fine to convert "a + b" into "a+b"
 		// but not to convert "a b" into "ab".
 		p.print(" ")
 	}
 
-	if sel.NestingSelector == css_ast.NestingSelectorPrefix {
-		p.print("&")
+	if sel.Combinator != 0 {
+		if !isFirst && !p.options.MinifyWhitespace {
+			p.print(" ")
+		}
+		p.css = append(p.css, sel.Combinator)
+		if !p.options.MinifyWhitespace {
+			p.print(" ")
+		}
 	}
 
-	if sel.Combinator != "" {
-		if !p.options.MinifyWhitespace {
-			p.print(" ")
-		}
-		p.print(sel.Combinator)
-		if !p.options.MinifyWhitespace {
-			p.print(" ")
-		}
+	if sel.HasNestingSelector {
+		p.print("&")
 	}
 
 	if sel.TypeSelector != nil {
@@ -439,12 +436,6 @@ func (p *printer) printCompoundSelector(sel css_ast.CompoundSelector, isFirst bo
 			p.printPseudoClassSelector(*s, whitespace)
 		}
 	}
-
-	// It doesn't matter where the "&" goes since all non-prefix cases are
-	// treated the same. This just always puts it as a suffix for simplicity.
-	if sel.NestingSelector == css_ast.NestingSelectorPresentButNotPrefix {
-		p.print("&")
-	}
 }
 
 func (p *printer) printNamespacedName(nsName css_ast.NamespacedName, whitespace trailingWhitespace) {
@@ -480,7 +471,8 @@ func (p *printer) printPseudoClassSelector(pseudo css_ast.SSPseudoClass, whitesp
 		p.print(":")
 	}
 
-	if len(pseudo.Args) > 0 {
+	// This checks for "nil" so we can distinguish ":is()" from ":is"
+	if pseudo.Args != nil {
 		p.printIdent(pseudo.Name, identNormal, canDiscardWhitespaceAfter)
 		p.print("(")
 		p.printTokens(pseudo.Args, printTokensOpts{})
@@ -814,6 +806,15 @@ func (p *printer) printTokens(tokens []css_ast.Token, opts printTokensOpts) bool
 			p.print(")")
 			p.recordImportPathForMetafile(t.ImportRecordIndex)
 
+		case css_lexer.TUnterminatedString:
+			// We must end this with a newline so that this string stays unterminated
+			p.print(t.Text)
+			p.print("\n")
+			if !p.options.MinifyWhitespace {
+				p.printIndent(opts.indent)
+			}
+			hasWhitespaceAfter = false
+
 		default:
 			p.print(t.Text)
 		}
