@@ -19,12 +19,13 @@ const (
 )
 
 type mockFS struct {
-	dirs  map[string]DirEntries
-	files map[string]string
-	Kind  MockKind
+	dirs          map[string]DirEntries
+	files         map[string]string
+	absWorkingDir string
+	Kind          MockKind
 }
 
-func MockFS(input map[string]string, kind MockKind) FS {
+func MockFS(input map[string]string, kind MockKind, absWorkingDir string) FS {
 	dirs := make(map[string]DirEntries)
 	files := make(map[string]string)
 
@@ -61,7 +62,7 @@ func MockFS(input map[string]string, kind MockKind) FS {
 		}
 	}
 
-	return &mockFS{dirs, files, kind}
+	return &mockFS{dirs, files, absWorkingDir, kind}
 }
 
 func (fs *mockFS) ReadDirectory(path string) (DirEntries, error, error) {
@@ -190,10 +191,7 @@ func (fs *mockFS) Join(parts ...string) string {
 }
 
 func (fs *mockFS) Cwd() string {
-	if fs.Kind == MockWindows {
-		return "C:\\"
-	}
-	return "/"
+	return fs.absWorkingDir
 }
 
 func splitOnSlash(path string) (string, string) {
