@@ -212,7 +212,10 @@ func (fs *realFS) ReadFile(path string) (contents string, canonicalError error,
 		data, ok := fs.watchData[path]
 		if canonicalError != nil {
 			data.state = stateFileMissing
-		} else if !ok {
+		} else if !ok || data.state == stateDirUnreadable {
+			// Note: If "ReadDirectory" is called before "ReadFile" with this same
+			// path, then "data.state" will be "stateDirUnreadable". In that case
+			// we want to transition to "stateFileNeedModKey" because it's a file.
 			data.state = stateFileNeedModKey
 		}
 		data.fileContents = fileContents
@@ -360,6 +363,14 @@ func (fs *realFS) readdir(dirname string) (entries []string, canonicalError erro
 	// Don't convert ENOTDIR to ENOENT here. ENOTDIR is a legitimate error
 	// condition for Readdirnames() on non-Windows platforms.
 
+	// Go's WebAssembly implementation returns EINVAL instead of ENOTDIR if we
+	// call "readdir" on a file. Canonicalize this to ENOTDIR so esbuild's path
+	// resolution code continues traversing instead of failing with an error.
+	// https://github.com/golang/go/blob/2449bbb5e614954ce9e99c8a481ea2ee73d72d61/src/syscall/fs_js.go#L144
+	if pathErr, ok := canonicalError.(*os.PathError); ok && pathErr.Unwrap() == syscall.EINVAL {
+		canonicalError = syscall.ENOTDIR
+	}
+
 	return entries, canonicalError, originalError
 }
 
