@@ -30,6 +30,11 @@ func (p *parser) skipTypeScriptBinding() {
 
 		// "[a, b]"
 		for p.lexer.Token != js_lexer.TCloseBracket {
+			// "[...a]"
+			if p.lexer.Token == js_lexer.TDotDotDot {
+				p.lexer.Next()
+			}
+
 			p.skipTypeScriptBinding()
 			if p.lexer.Token != js_lexer.TComma {
 				break
@@ -286,12 +291,12 @@ loop:
 				return
 			}
 
-			p.skipTypeScriptTypeParameters(typeParametersNormal)
+			p.skipTypeScriptTypeParameters(allowConstModifier)
 			p.skipTypeScriptParenOrFnType()
 
 		case js_lexer.TLessThan:
 			// "<T>() => Foo<T>"
-			p.skipTypeScriptTypeParameters(typeParametersNormal)
+			p.skipTypeScriptTypeParameters(allowConstModifier)
 			p.skipTypeScriptParenOrFnType()
 
 		case js_lexer.TOpenParen:
@@ -309,11 +314,12 @@ loop:
 				// Valid:
 				//   "[keyof: string]"
 				//   "{[keyof: string]: number}"
+				//   "{[keyof in string]: number}"
 				//
 				// Invalid:
 				//   "A extends B ? keyof : string"
 				//
-				if p.lexer.Token != js_lexer.TColon || (!flags.has(isIndexSignatureFlag) && !flags.has(allowTupleLabelsFlag)) {
+				if (p.lexer.Token != js_lexer.TColon && p.lexer.Token != js_lexer.TIn) || (!flags.has(isIndexSignatureFlag) && !flags.has(allowTupleLabelsFlag)) {
 					p.skipTypeScriptType(js_ast.LPrefix)
 				}
 				break loop
@@ -324,7 +330,8 @@ loop:
 				// "type Foo = Bar extends [infer T] ? T : null"
 				// "type Foo = Bar extends [infer T extends string] ? T : null"
 				// "type Foo = Bar extends [infer T extends string ? infer T : never] ? T : null"
-				if p.lexer.Token != js_lexer.TColon || (!flags.has(isIndexSignatureFlag) && !flags.has(allowTupleLabelsFlag)) {
+				// "type Foo = { [infer in Bar]: number }"
+				if (p.lexer.Token != js_lexer.TColon && p.lexer.Token != js_lexer.TIn) || (!flags.has(isIndexSignatureFlag) && !flags.has(allowTupleLabelsFlag)) {
 					p.lexer.Expect(js_lexer.TIdentifier)
 					if p.lexer.Token == js_lexer.TExtends {
 						p.trySkipTypeScriptConstraintOfInferTypeWithBacktracking(flags)
@@ -375,7 +382,7 @@ loop:
 
 			// "let foo: any \n <number>foo" must not become a single type
 			if checkTypeParameters && !p.lexer.HasNewlineBefore {
-				p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */)
+				p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{})
 			}
 
 		case js_lexer.TTypeof:
@@ -407,7 +414,7 @@ loop:
 				}
 
 				if !p.lexer.HasNewlineBefore {
-					p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */)
+					p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{})
 				}
 			}
 
@@ -503,7 +510,7 @@ loop:
 
 			// "{ <A extends B>(): c.d \n <E extends F>(): g.h }" must not become a single type
 			if !p.lexer.HasNewlineBefore {
-				p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */)
+				p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{})
 			}
 
 		case js_lexer.TOpenBracket:
@@ -594,7 +601,7 @@ func (p *parser) skipTypeScriptObjectType() {
 		}
 
 		// Type parameters come right after the optional mark
-		p.skipTypeScriptTypeParameters(typeParametersNormal)
+		p.skipTypeScriptTypeParameters(allowConstModifier)
 
 		switch p.lexer.Token {
 		case js_lexer.TColon:
@@ -635,106 +642,144 @@ func (p *parser) skipTypeScriptObjectType() {
 	p.lexer.Expect(js_lexer.TCloseBrace)
 }
 
-type typeParameters uint8
+type typeParameterFlags uint8
 
 const (
-	typeParametersNormal typeParameters = iota
-	typeParametersWithInOutVarianceAnnotations
+	// TypeScript 4.7
+	allowInOutVarianceAnnotations typeParameterFlags = 1 << iota
+
+	// TypeScript 5.0
+	allowConstModifier
+)
+
+type skipTypeScriptTypeParametersResult uint8
+
+const (
+	didNotSkipAnything skipTypeScriptTypeParametersResult = iota
+	couldBeTypeCast
+	definitelyTypeParameters
 )
 
 // This is the type parameter declarations that go with other symbol
 // declarations (class, function, type, etc.)
-func (p *parser) skipTypeScriptTypeParameters(mode typeParameters) {
-	if p.lexer.Token == js_lexer.TLessThan {
-		p.lexer.Next()
+func (p *parser) skipTypeScriptTypeParameters(flags typeParameterFlags) skipTypeScriptTypeParametersResult {
+	if p.lexer.Token != js_lexer.TLessThan {
+		return didNotSkipAnything
+	}
 
+	p.lexer.Next()
+	result := couldBeTypeCast
+
+	for {
+		hasIn := false
+		hasOut := false
+		expectIdentifier := true
+		invalidModifierRange := logger.Range{}
+
+		// Scan over a sequence of "in" and "out" modifiers (a.k.a. optional
+		// variance annotations) as well as "const" modifiers
 		for {
-			hasIn := false
-			hasOut := false
-			expectIdentifier := true
-			invalidModifierRange := logger.Range{}
-
-			// Scan over a sequence of "in" and "out" modifiers (a.k.a. optional variance annotations)
-			for {
-				if p.lexer.Token == js_lexer.TIn {
-					if invalidModifierRange.Len == 0 && (mode != typeParametersWithInOutVarianceAnnotations || hasIn || hasOut) {
-						// Valid:
-						//   "type Foo<in T> = T"
-						// Invalid:
-						//   "type Foo<in in T> = T"
-						//   "type Foo<out in T> = T"
-						invalidModifierRange = p.lexer.Range()
-					}
-					p.lexer.Next()
-					hasIn = true
-					expectIdentifier = true
-					continue
+			if p.lexer.Token == js_lexer.TConst {
+				if invalidModifierRange.Len == 0 && (flags&allowConstModifier) == 0 {
+					// Valid:
+					//   "class Foo<const T> {}"
+					// Invalid:
+					//   "interface Foo<const T> {}"
+					invalidModifierRange = p.lexer.Range()
 				}
+				result = definitelyTypeParameters
+				p.lexer.Next()
+				expectIdentifier = true
+				continue
+			}
 
-				if p.lexer.IsContextualKeyword("out") {
-					r := p.lexer.Range()
-					if invalidModifierRange.Len == 0 && mode != typeParametersWithInOutVarianceAnnotations {
-						invalidModifierRange = r
-					}
-					p.lexer.Next()
-					if invalidModifierRange.Len == 0 && hasOut && (p.lexer.Token == js_lexer.TIn || p.lexer.Token == js_lexer.TIdentifier) {
-						// Valid:
-						//   "type Foo<out T> = T"
-						//   "type Foo<out out> = T"
-						//   "type Foo<out out, T> = T"
-						//   "type Foo<out out = T> = T"
-						//   "type Foo<out out extends T> = T"
-						// Invalid:
-						//   "type Foo<out out in T> = T"
-						//   "type Foo<out out T> = T"
-						invalidModifierRange = r
-					}
-					hasOut = true
-					expectIdentifier = false
-					continue
+			if p.lexer.Token == js_lexer.TIn {
+				if invalidModifierRange.Len == 0 && ((flags&allowInOutVarianceAnnotations) == 0 || hasIn || hasOut) {
+					// Valid:
+					//   "type Foo<in T> = T"
+					// Invalid:
+					//   "type Foo<in in T> = T"
+					//   "type Foo<out in T> = T"
+					invalidModifierRange = p.lexer.Range()
 				}
-
-				break
-			}
-
-			// Only report an error for the first invalid modifier
-			if invalidModifierRange.Len > 0 {
-				p.log.AddError(&p.tracker, invalidModifierRange, fmt.Sprintf(
-					"The modifier %q is not valid here:", p.source.TextForRange(invalidModifierRange)))
-			}
-
-			// expectIdentifier => Mandatory identifier (e.g. after "type Foo <in ___")
-			// !expectIdentifier => Optional identifier (e.g. after "type Foo <out ___" since "out" may be the identifier)
-			if expectIdentifier || p.lexer.Token == js_lexer.TIdentifier {
-				p.lexer.Expect(js_lexer.TIdentifier)
-			}
-
-			// "class Foo<T extends number> {}"
-			if p.lexer.Token == js_lexer.TExtends {
 				p.lexer.Next()
-				p.skipTypeScriptType(js_ast.LLowest)
+				hasIn = true
+				expectIdentifier = true
+				continue
 			}
 
-			// "class Foo<T = void> {}"
-			if p.lexer.Token == js_lexer.TEquals {
+			if p.lexer.IsContextualKeyword("out") {
+				r := p.lexer.Range()
+				if invalidModifierRange.Len == 0 && (flags&allowInOutVarianceAnnotations) == 0 {
+					invalidModifierRange = r
+				}
 				p.lexer.Next()
-				p.skipTypeScriptType(js_ast.LLowest)
+				if invalidModifierRange.Len == 0 && hasOut && (p.lexer.Token == js_lexer.TIn || p.lexer.Token == js_lexer.TIdentifier) {
+					// Valid:
+					//   "type Foo<out T> = T"
+					//   "type Foo<out out> = T"
+					//   "type Foo<out out, T> = T"
+					//   "type Foo<out out = T> = T"
+					//   "type Foo<out out extends T> = T"
+					// Invalid:
+					//   "type Foo<out out in T> = T"
+					//   "type Foo<out out T> = T"
+					invalidModifierRange = r
+				}
+				hasOut = true
+				expectIdentifier = false
+				continue
 			}
 
-			if p.lexer.Token != js_lexer.TComma {
-				break
-			}
-			p.lexer.Next()
-			if p.lexer.Token == js_lexer.TGreaterThan {
-				break
-			}
+			break
 		}
 
-		p.lexer.ExpectGreaterThan(false /* isInsideJSXElement */)
+		// Only report an error for the first invalid modifier
+		if invalidModifierRange.Len > 0 {
+			p.log.AddError(&p.tracker, invalidModifierRange, fmt.Sprintf(
+				"The modifier %q is not valid here:", p.source.TextForRange(invalidModifierRange)))
+		}
+
+		// expectIdentifier => Mandatory identifier (e.g. after "type Foo <in ___")
+		// !expectIdentifier => Optional identifier (e.g. after "type Foo <out ___" since "out" may be the identifier)
+		if expectIdentifier || p.lexer.Token == js_lexer.TIdentifier {
+			p.lexer.Expect(js_lexer.TIdentifier)
+		}
+
+		// "class Foo<T extends number> {}"
+		if p.lexer.Token == js_lexer.TExtends {
+			result = definitelyTypeParameters
+			p.lexer.Next()
+			p.skipTypeScriptType(js_ast.LLowest)
+		}
+
+		// "class Foo<T = void> {}"
+		if p.lexer.Token == js_lexer.TEquals {
+			result = definitelyTypeParameters
+			p.lexer.Next()
+			p.skipTypeScriptType(js_ast.LLowest)
+		}
+
+		if p.lexer.Token != js_lexer.TComma {
+			break
+		}
+		p.lexer.Next()
+		if p.lexer.Token == js_lexer.TGreaterThan {
+			result = definitelyTypeParameters
+			break
+		}
 	}
+
+	p.lexer.ExpectGreaterThan(false /* isInsideJSXElement */)
+	return result
 }
 
-func (p *parser) skipTypeScriptTypeArguments(isInsideJSXElement bool) bool {
+type skipTypeScriptTypeArgumentsOpts struct {
+	isInsideJSXElement               bool
+	isParseTypeArgumentsInExpression bool
+}
+
+func (p *parser) skipTypeScriptTypeArguments(opts skipTypeScriptTypeArgumentsOpts) bool {
 	switch p.lexer.Token {
 	case js_lexer.TLessThan, js_lexer.TLessThanEquals,
 		js_lexer.TLessThanLessThan, js_lexer.TLessThanLessThanEquals:
@@ -753,11 +798,31 @@ func (p *parser) skipTypeScriptTypeArguments(isInsideJSXElement bool) bool {
 	}
 
 	// This type argument list must end with a ">"
-	p.lexer.ExpectGreaterThan(isInsideJSXElement)
+	if !opts.isParseTypeArgumentsInExpression {
+		// Normally TypeScript allows any token starting with ">". For example,
+		// "Array<Array<number>>()" is a type argument list even though there's a
+		// ">>" token, because ">>" starts with ">".
+		p.lexer.ExpectGreaterThan(opts.isInsideJSXElement)
+	} else {
+		// However, if we're emulating the TypeScript compiler's function called
+		// "parseTypeArgumentsInExpression" function, then we must only allow the
+		// ">" token itself. For example, "x < y >= z" is not a type argument list.
+		//
+		// This doesn't detect ">>" in "Array<Array<number>>()" because the inner
+		// type argument list isn't a call to "parseTypeArgumentsInExpression"
+		// because it's within a type context, not an expression context. So the
+		// token that we see here is ">" in that case because the first ">" has
+		// already been stripped off of the ">>" by the inner call.
+		if opts.isInsideJSXElement {
+			p.lexer.ExpectInsideJSXElement(js_lexer.TGreaterThan)
+		} else {
+			p.lexer.Expect(js_lexer.TGreaterThan)
+		}
+	}
 	return true
 }
 
-func (p *parser) trySkipTypeScriptTypeArgumentsWithBacktracking() bool {
+func (p *parser) trySkipTypeArgumentsInExpressionWithBacktracking() bool {
 	oldLexer := p.lexer
 	p.lexer.IsLogDisabled = true
 
@@ -771,9 +836,9 @@ func (p *parser) trySkipTypeScriptTypeArgumentsWithBacktracking() bool {
 		}
 	}()
 
-	if p.skipTypeScriptTypeArguments(false /* isInsideJSXElement */) {
+	if p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{isParseTypeArgumentsInExpression: true}) {
 		// Check the token after the type argument list and backtrack if it's invalid
-		if !p.canFollowTypeArgumentsInExpression() {
+		if !p.tsCanFollowTypeArgumentsInExpression() {
 			p.lexer.Unexpected()
 		}
 	}
@@ -784,7 +849,7 @@ func (p *parser) trySkipTypeScriptTypeArgumentsWithBacktracking() bool {
 	return true
 }
 
-func (p *parser) trySkipTypeScriptTypeParametersThenOpenParenWithBacktracking() bool {
+func (p *parser) trySkipTypeScriptTypeParametersThenOpenParenWithBacktracking() skipTypeScriptTypeParametersResult {
 	oldLexer := p.lexer
 	p.lexer.IsLogDisabled = true
 
@@ -798,7 +863,7 @@ func (p *parser) trySkipTypeScriptTypeParametersThenOpenParenWithBacktracking()
 		}
 	}()
 
-	p.skipTypeScriptTypeParameters(typeParametersNormal)
+	result := p.skipTypeScriptTypeParameters(allowConstModifier)
 	if p.lexer.Token != js_lexer.TOpenParen {
 		p.lexer.Unexpected()
 	}
@@ -806,7 +871,7 @@ func (p *parser) trySkipTypeScriptTypeParametersThenOpenParenWithBacktracking()
 	// Restore the log disabled flag. Note that we can't just set it back to false
 	// because it may have been true to start with.
 	p.lexer.IsLogDisabled = oldLexer.IsLogDisabled
-	return true
+	return result
 }
 
 func (p *parser) trySkipTypeScriptArrowReturnTypeWithBacktracking() bool {
@@ -893,13 +958,16 @@ func (p *parser) isTSArrowFnJSX() (isTSArrowFn bool) {
 	p.lexer.Next()
 
 	// Look ahead to see if this should be an arrow function instead
+	if p.lexer.Token == js_lexer.TConst {
+		p.lexer.Next()
+	}
 	if p.lexer.Token == js_lexer.TIdentifier {
 		p.lexer.Next()
 		if p.lexer.Token == js_lexer.TComma || p.lexer.Token == js_lexer.TEquals {
 			isTSArrowFn = true
 		} else if p.lexer.Token == js_lexer.TExtends {
 			p.lexer.Next()
-			isTSArrowFn = p.lexer.Token != js_lexer.TEquals && p.lexer.Token != js_lexer.TGreaterThan
+			isTSArrowFn = p.lexer.Token != js_lexer.TEquals && p.lexer.Token != js_lexer.TGreaterThan && p.lexer.Token != js_lexer.TSlash
 		}
 	}
 
@@ -908,7 +976,168 @@ func (p *parser) isTSArrowFnJSX() (isTSArrowFn bool) {
 	return
 }
 
-func (p *parser) nextTokenIsOpenParenOrLessThanOrDot() (result bool) {
+// This function is taken from the official TypeScript compiler source code:
+// https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
+//
+// This function is pretty inefficient as written, and could be collapsed into
+// a single switch statement. But that would make it harder to keep this in
+// sync with the TypeScript compiler's source code, so we keep doing it the
+// slow way.
+func (p *parser) tsCanFollowTypeArgumentsInExpression() bool {
+	switch p.lexer.Token {
+	case
+		// These tokens can follow a type argument list in a call expression.
+		js_lexer.TOpenParen,                     // foo<x>(
+		js_lexer.TNoSubstitutionTemplateLiteral, // foo<T> `...`
+		js_lexer.TTemplateHead:                  // foo<T> `...${100}...`
+		return true
+
+	// A type argument list followed by `<` never makes sense, and a type argument list followed
+	// by `>` is ambiguous with a (re-scanned) `>>` operator, so we disqualify both. Also, in
+	// this context, `+` and `-` are unary operators, not binary operators.
+	case js_lexer.TLessThan,
+		js_lexer.TGreaterThan,
+		js_lexer.TPlus,
+		js_lexer.TMinus,
+		// TypeScript always sees "TGreaterThan" instead of these tokens since
+		// their scanner works a little differently than our lexer. So since
+		// "TGreaterThan" is forbidden above, we also forbid these too.
+		js_lexer.TGreaterThanEquals,
+		js_lexer.TGreaterThanGreaterThan,
+		js_lexer.TGreaterThanGreaterThanEquals,
+		js_lexer.TGreaterThanGreaterThanGreaterThan,
+		js_lexer.TGreaterThanGreaterThanGreaterThanEquals:
+		return false
+	}
+
+	// We favor the type argument list interpretation when it is immediately followed by
+	// a line break, a binary operator, or something that can't start an expression.
+	return p.lexer.HasNewlineBefore || p.tsIsBinaryOperator() || !p.tsIsStartOfExpression()
+}
+
+// This function is taken from the official TypeScript compiler source code:
+// https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
+func (p *parser) tsIsBinaryOperator() bool {
+	switch p.lexer.Token {
+	case js_lexer.TIn:
+		return p.allowIn
+
+	case
+		js_lexer.TQuestionQuestion,
+		js_lexer.TBarBar,
+		js_lexer.TAmpersandAmpersand,
+		js_lexer.TBar,
+		js_lexer.TCaret,
+		js_lexer.TAmpersand,
+		js_lexer.TEqualsEquals,
+		js_lexer.TExclamationEquals,
+		js_lexer.TEqualsEqualsEquals,
+		js_lexer.TExclamationEqualsEquals,
+		js_lexer.TLessThan,
+		js_lexer.TGreaterThan,
+		js_lexer.TLessThanEquals,
+		js_lexer.TGreaterThanEquals,
+		js_lexer.TInstanceof,
+		js_lexer.TLessThanLessThan,
+		js_lexer.TGreaterThanGreaterThan,
+		js_lexer.TGreaterThanGreaterThanGreaterThan,
+		js_lexer.TPlus,
+		js_lexer.TMinus,
+		js_lexer.TAsterisk,
+		js_lexer.TSlash,
+		js_lexer.TPercent,
+		js_lexer.TAsteriskAsterisk:
+		return true
+
+	case js_lexer.TIdentifier:
+		if p.lexer.IsContextualKeyword("as") || p.lexer.IsContextualKeyword("satisfies") {
+			return true
+		}
+	}
+
+	return false
+}
+
+// This function is taken from the official TypeScript compiler source code:
+// https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
+func (p *parser) tsIsStartOfExpression() bool {
+	if p.tsIsStartOfLeftHandSideExpression() {
+		return true
+	}
+
+	switch p.lexer.Token {
+	case
+		js_lexer.TPlus,
+		js_lexer.TMinus,
+		js_lexer.TTilde,
+		js_lexer.TExclamation,
+		js_lexer.TDelete,
+		js_lexer.TTypeof,
+		js_lexer.TVoid,
+		js_lexer.TPlusPlus,
+		js_lexer.TMinusMinus,
+		js_lexer.TLessThan,
+		js_lexer.TPrivateIdentifier,
+		js_lexer.TAt:
+		return true
+
+	default:
+		if p.lexer.Token == js_lexer.TIdentifier && (p.lexer.Identifier.String == "await" || p.lexer.Identifier.String == "yield") {
+			// Yield/await always starts an expression.  Either it is an identifier (in which case
+			// it is definitely an expression).  Or it's a keyword (either because we're in
+			// a generator or async function, or in strict mode (or both)) and it started a yield or await expression.
+			return true
+		}
+
+		// Error tolerance.  If we see the start of some binary operator, we consider
+		// that the start of an expression.  That way we'll parse out a missing identifier,
+		// give a good message about an identifier being missing, and then consume the
+		// rest of the binary expression.
+		if p.tsIsBinaryOperator() {
+			return true
+		}
+
+		return p.tsIsIdentifier()
+	}
+}
+
+// This function is taken from the official TypeScript compiler source code:
+// https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
+func (p *parser) tsIsStartOfLeftHandSideExpression() bool {
+	switch p.lexer.Token {
+	case
+		js_lexer.TThis,
+		js_lexer.TSuper,
+		js_lexer.TNull,
+		js_lexer.TTrue,
+		js_lexer.TFalse,
+		js_lexer.TNumericLiteral,
+		js_lexer.TBigIntegerLiteral,
+		js_lexer.TStringLiteral,
+		js_lexer.TNoSubstitutionTemplateLiteral,
+		js_lexer.TTemplateHead,
+		js_lexer.TOpenParen,
+		js_lexer.TOpenBracket,
+		js_lexer.TOpenBrace,
+		js_lexer.TFunction,
+		js_lexer.TClass,
+		js_lexer.TNew,
+		js_lexer.TSlash,
+		js_lexer.TSlashEquals,
+		js_lexer.TIdentifier:
+		return true
+
+	case js_lexer.TImport:
+		return p.tsLookAheadNextTokenIsOpenParenOrLessThanOrDot()
+
+	default:
+		return p.tsIsIdentifier()
+	}
+}
+
+// This function is taken from the official TypeScript compiler source code:
+// https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
+func (p *parser) tsLookAheadNextTokenIsOpenParenOrLessThanOrDot() (result bool) {
 	oldLexer := p.lexer
 	p.lexer.Next()
 
@@ -923,85 +1152,24 @@ func (p *parser) nextTokenIsOpenParenOrLessThanOrDot() (result bool) {
 
 // This function is taken from the official TypeScript compiler source code:
 // https://github.com/microsoft/TypeScript/blob/master/src/compiler/parser.ts
-func (p *parser) canFollowTypeArgumentsInExpression() bool {
-	switch p.lexer.Token {
-	case
-		// These tokens can follow a type argument list in a call expression.
-		js_lexer.TOpenParen,                     // foo<x>(
-		js_lexer.TNoSubstitutionTemplateLiteral, // foo<T> `...`
-		js_lexer.TTemplateHead:                  // foo<T> `...${100}...`
-		return true
+func (p *parser) tsIsIdentifier() bool {
+	if p.lexer.Token == js_lexer.TIdentifier {
+		// If we have a 'yield' keyword, and we're in the [yield] context, then 'yield' is
+		// considered a keyword and is not an identifier.
+		if p.fnOrArrowDataParse.yield != allowIdent && p.lexer.Identifier.String == "yield" {
+			return false
+		}
 
-	// Consider something a type argument list only if the following token can't start an expression.
-	case
-		// From "isStartOfExpression()"
-		js_lexer.TPlus,
-		js_lexer.TMinus,
-		js_lexer.TTilde,
-		js_lexer.TExclamation,
-		js_lexer.TDelete,
-		js_lexer.TTypeof,
-		js_lexer.TVoid,
-		js_lexer.TPlusPlus,
-		js_lexer.TMinusMinus,
-		js_lexer.TLessThan,
+		// If we have a 'await' keyword, and we're in the [Await] context, then 'await' is
+		// considered a keyword and is not an identifier.
+		if p.fnOrArrowDataParse.await != allowIdent && p.lexer.Identifier.String == "await" {
+			return false
+		}
 
-		// From "isStartOfLeftHandSideExpression()"
-		js_lexer.TThis,
-		js_lexer.TSuper,
-		js_lexer.TNull,
-		js_lexer.TTrue,
-		js_lexer.TFalse,
-		js_lexer.TNumericLiteral,
-		js_lexer.TBigIntegerLiteral,
-		js_lexer.TStringLiteral,
-		js_lexer.TOpenBracket,
-		js_lexer.TOpenBrace,
-		js_lexer.TFunction,
-		js_lexer.TClass,
-		js_lexer.TNew,
-		js_lexer.TSlash,
-		js_lexer.TSlashEquals,
-		js_lexer.TIdentifier,
-
-		// From "isBinaryOperator()"
-		js_lexer.TQuestionQuestion,
-		js_lexer.TBarBar,
-		js_lexer.TAmpersandAmpersand,
-		js_lexer.TBar,
-		js_lexer.TCaret,
-		js_lexer.TAmpersand,
-		js_lexer.TEqualsEquals,
-		js_lexer.TExclamationEquals,
-		js_lexer.TEqualsEqualsEquals,
-		js_lexer.TExclamationEqualsEquals,
-		js_lexer.TGreaterThan,
-		js_lexer.TLessThanEquals,
-		js_lexer.TGreaterThanEquals,
-		js_lexer.TInstanceof,
-		js_lexer.TLessThanLessThan,
-		js_lexer.TGreaterThanGreaterThan,
-		js_lexer.TGreaterThanGreaterThanGreaterThan,
-		js_lexer.TAsterisk,
-		js_lexer.TPercent,
-		js_lexer.TAsteriskAsterisk,
-
-		// TypeScript always sees "TGreaterThan" instead of these tokens since
-		// their scanner works a little differently than our lexer. So since
-		// "TGreaterThan" is forbidden above, we also forbid these too.
-		js_lexer.TGreaterThanGreaterThanEquals,
-		js_lexer.TGreaterThanGreaterThanGreaterThanEquals:
-		return false
-
-	case js_lexer.TIn:
-		return !p.allowIn
-
-	case js_lexer.TImport:
-		return !p.nextTokenIsOpenParenOrLessThanOrDot()
-
-	default:
 		return true
 	}
+
+	return false
 }
 
 func (p *parser) skipTypeScriptInterfaceStmt(opts parseStmtOpts) {
@@ -1012,7 +1180,7 @@ func (p *parser) skipTypeScriptInterfaceStmt(opts parseStmtOpts) {
 		p.localTypeNames[name] = true
 	}
 
-	p.skipTypeScriptTypeParameters(typeParametersWithInOutVarianceAnnotations)
+	p.skipTypeScriptTypeParameters(allowInOutVarianceAnnotations)
 
 	if p.lexer.Token == js_lexer.TExtends {
 		p.lexer.Next()
@@ -1087,73 +1255,12 @@ func (p *parser) skipTypeScriptTypeStmt(opts parseStmtOpts) {
 		p.localTypeNames[name] = true
 	}
 
-	p.skipTypeScriptTypeParameters(typeParametersWithInOutVarianceAnnotations)
+	p.skipTypeScriptTypeParameters(allowInOutVarianceAnnotations)
 	p.lexer.Expect(js_lexer.TEquals)
 	p.skipTypeScriptType(js_ast.LLowest)
 	p.lexer.ExpectOrInsertSemicolon()
 }
 
-func (p *parser) parseTypeScriptDecorators(tsDecoratorScope *js_ast.Scope) []js_ast.Expr {
-	var tsDecorators []js_ast.Expr
-
-	if p.options.ts.Parse {
-		// TypeScript decorators cause us to temporarily revert to the scope that
-		// encloses the class declaration, since that's where the generated code
-		// for TypeScript decorators will be inserted.
-		oldScope := p.currentScope
-		p.currentScope = tsDecoratorScope
-
-		for p.lexer.Token == js_lexer.TAt {
-			loc := p.lexer.Loc()
-			p.lexer.Next()
-
-			// Parse a new/call expression with "exprFlagTSDecorator" so we ignore
-			// EIndex expressions, since they may be part of a computed property:
-			//
-			//   class Foo {
-			//     @foo ['computed']() {}
-			//   }
-			//
-			// This matches the behavior of the TypeScript compiler.
-			value := p.parseExprWithFlags(js_ast.LNew, exprFlagTSDecorator)
-			value.Loc = loc
-			tsDecorators = append(tsDecorators, value)
-		}
-
-		// Avoid "popScope" because this decorator scope is not hierarchical
-		p.currentScope = oldScope
-	}
-
-	return tsDecorators
-}
-
-func (p *parser) logInvalidDecoratorError(classKeyword logger.Range) {
-	if p.options.ts.Parse && p.lexer.Token == js_lexer.TAt {
-		// Forbid decorators inside class expressions
-		p.lexer.AddRangeErrorWithNotes(p.lexer.Range(), "Decorators can only be used with class declarations in TypeScript",
-			[]logger.MsgData{p.tracker.MsgData(classKeyword, "This is a class expression, not a class declaration:")})
-
-		// Parse and discard decorators for error recovery
-		scopeIndex := len(p.scopesInOrder)
-		p.parseTypeScriptDecorators(p.currentScope)
-		p.discardScopesUpTo(scopeIndex)
-	}
-}
-
-func (p *parser) logMisplacedDecoratorError(tsDecorators *deferredTSDecorators) {
-	found := fmt.Sprintf("%q", p.lexer.Raw())
-	if p.lexer.Token == js_lexer.TEndOfFile {
-		found = "end of file"
-	}
-
-	// Try to be helpful by pointing out the decorator
-	p.lexer.AddRangeErrorWithNotes(p.lexer.Range(), fmt.Sprintf("Expected \"class\" after TypeScript decorator but found %s", found), []logger.MsgData{
-		p.tracker.MsgData(logger.Range{Loc: tsDecorators.values[0].Loc}, "The preceding TypeScript decorator is here:"),
-		{Text: "Decorators can only be used with class declarations in TypeScript."},
-	})
-	p.discardScopesUpTo(tsDecorators.scopeIndex)
-}
-
 func (p *parser) parseTypeScriptEnumStmt(loc logger.Loc, opts parseStmtOpts) js_ast.Stmt {
 	p.lexer.Expect(js_lexer.TEnum)
 	nameLoc := p.lexer.Loc()
@@ -1173,6 +1280,7 @@ func (p *parser) parseTypeScriptEnumStmt(loc logger.Loc, opts parseStmtOpts) js_
 	}
 
 	// Declare the enum and create the scope
+	scopeIndex := len(p.scopesInOrder)
 	if !opts.isTypeScriptDeclare {
 		name.Ref = p.declareSymbol(js_ast.SymbolTSEnum, nameLoc, nameText)
 		p.pushScopeForParsePass(js_ast.ScopeEntry, loc)
@@ -1315,6 +1423,12 @@ func (p *parser) parseTypeScriptEnumStmt(loc logger.Loc, opts parseStmtOpts) js_
 		return js_ast.Stmt{Loc: loc, Data: &js_ast.STypeScript{}}
 	}
 
+	// Save these for when we do out-of-order enum visiting
+	if p.scopesInOrderForEnum == nil {
+		p.scopesInOrderForEnum = make(map[logger.Loc][]scopeOrder)
+	}
+	p.scopesInOrderForEnum[loc] = p.scopesInOrder[scopeIndex:]
+
 	return js_ast.Stmt{Loc: loc, Data: &js_ast.SEnum{
 		Name:     name,
 		Arg:      tsNamespace.ArgRef,
