@@ -1,6 +1,7 @@
 package js_printer
 
 import (
+	"strings"
 	"testing"
 
 	"github.com/evanw/esbuild/internal/compat"
@@ -278,6 +279,18 @@ func TestNumber(t *testing.T) {
 	expectPrintedMinify(t, "x = 0xFFFF_FFFF_FFFF_F000", "x=0xfffffffffffff000;")
 	expectPrintedMinify(t, "x = 0xFFFF_FFFF_FFFF_F800", "x=1844674407370955e4;")
 	expectPrintedMinify(t, "x = 0xFFFF_FFFF_FFFF_FFFF", "x=18446744073709552e3;")
+
+	// Check printing a space in between a number and a subsequent "."
+	expectPrintedMinify(t, "x = 0.0001 .y", "x=1e-4.y;")
+	expectPrintedMinify(t, "x = 0.001 .y", "x=.001.y;")
+	expectPrintedMinify(t, "x = 0.01 .y", "x=.01.y;")
+	expectPrintedMinify(t, "x = 0.1 .y", "x=.1.y;")
+	expectPrintedMinify(t, "x = 0 .y", "x=0 .y;")
+	expectPrintedMinify(t, "x = 10 .y", "x=10 .y;")
+	expectPrintedMinify(t, "x = 100 .y", "x=100 .y;")
+	expectPrintedMinify(t, "x = 1000 .y", "x=1e3.y;")
+	expectPrintedMinify(t, "x = 12345 .y", "x=12345 .y;")
+	expectPrintedMinify(t, "x = 0xFFFF_0000_FFFF_0000 .y", "x=0xffff0000ffff0000.y;")
 }
 
 func TestArray(t *testing.T) {
@@ -780,6 +793,8 @@ func TestWhitespace(t *testing.T) {
 
 	expectPrintedMinify(t, "x in function(){}", "x in function(){};")
 	expectPrintedMinify(t, "x instanceof function(){}", "x instanceof function(){};")
+	expectPrintedMinify(t, "π in function(){}", "π in function(){};")
+	expectPrintedMinify(t, "π instanceof function(){}", "π instanceof function(){};")
 
 	expectPrintedMinify(t, "()=>({})", "()=>({});")
 	expectPrintedMinify(t, "()=>({}[1])", "()=>({})[1];")
@@ -1072,3 +1087,12 @@ func TestInfinity(t *testing.T) {
 	expectPrintedMangleMinify(t, "x = y / Infinity", "x=y/(1/0);")
 	expectPrintedMangleMinify(t, "throw Infinity", "throw 1/0;")
 }
+
+func TestBinaryOperatorVisitor(t *testing.T) {
+	// Make sure the inner "/*b*/" comment doesn't disappear due to weird binary visitor stuff
+	expectPrintedMangle(t, "x = (0, /*a*/ (0, /*b*/ (0, /*c*/ 1 == 2) + 3) * 4)", "x = /*a*/\n/*b*/\n(/*c*/\n!1 + 3) * 4;\n")
+
+	// Make sure deeply-nested ASTs don't cause a stack overflow
+	x := "x = f()" + strings.Repeat(" || f()", 10_000) + ";\n"
+	expectPrinted(t, x, x)
+}
