@@ -600,6 +600,51 @@ func validateDefines(
 		// Define simple expressions
 		if defineExpr.Constant != nil || len(defineExpr.Parts) > 0 {
 			rawDefines[key] = config.DefineData{DefineExpr: &defineExpr}
+
+			// Try to be helpful for common mistakes
+			if len(defineExpr.Parts) == 1 && key == "process.env.NODE_ENV" {
+				data := logger.MsgData{
+					Text: fmt.Sprintf("%q is defined as an identifier instead of a string (surround %q with quotes to get a string)", key, value),
+				}
+				part := defineExpr.Parts[0]
+
+				switch logger.API {
+				case logger.CLIAPI:
+					data.Location = &logger.MsgLocation{
+						File:       "<cli>",
+						Line:       1,
+						Column:     30,
+						Length:     len(part),
+						LineText:   fmt.Sprintf("--define:process.env.NODE_ENV=%s", part),
+						Suggestion: fmt.Sprintf("\\\"%s\\\"", part),
+					}
+
+				case logger.JSAPI:
+					data.Location = &logger.MsgLocation{
+						File:       "<js>",
+						Line:       1,
+						Column:     34,
+						Length:     len(part) + 2,
+						LineText:   fmt.Sprintf("define: { 'process.env.NODE_ENV': '%s' }", part),
+						Suggestion: fmt.Sprintf("'\"%s\"'", part),
+					}
+
+				case logger.GoAPI:
+					data.Location = &logger.MsgLocation{
+						File:       "<go>",
+						Line:       1,
+						Column:     50,
+						Length:     len(part) + 2,
+						LineText:   fmt.Sprintf("Define: map[string]string{\"process.env.NODE_ENV\": \"%s\"}", part),
+						Suggestion: fmt.Sprintf("\"\\\"%s\\\"\"", part),
+					}
+				}
+
+				log.AddMsgID(logger.MsgID_JS_SuspiciousDefine, logger.Msg{
+					Kind: logger.Warning,
+					Data: data,
+				})
+			}
 			continue
 		}
 
@@ -858,7 +903,6 @@ func contextImpl(buildOpts BuildOptions) (*internalContext, []Message) {
 		LogLevel:      validateLogLevel(buildOpts.LogLevel),
 		Overrides:     validateLogOverrides(buildOpts.LogOverride),
 	}
-	log := logger.NewStderrLog(logOptions)
 
 	// Validate that the current working directory is an absolute path
 	absWorkingDir := buildOpts.AbsWorkingDir
@@ -871,6 +915,7 @@ func contextImpl(buildOpts BuildOptions) (*internalContext, []Message) {
 		DoNotCache: true,
 	})
 	if err != nil {
+		log := logger.NewStderrLog(logOptions)
 		log.AddError(nil, logger.Range{}, err.Error())
 		return nil, convertMessagesToPublic(logger.Error, log.Done())
 	}
@@ -879,7 +924,8 @@ func contextImpl(buildOpts BuildOptions) (*internalContext, []Message) {
 	// directory doesn't change, since breaking that invariant would break the
 	// validation that we just did above.
 	caches := cache.MakeCacheSet()
-	onEndCallbacks, finalizeBuildOptions := loadPlugins(&buildOpts, realFS, log, caches)
+	log := logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug, logOptions.Overrides)
+	onEndCallbacks, onDisposeCallbacks, finalizeBuildOptions := loadPlugins(&buildOpts, realFS, log, caches)
 	options, entryPoints := validateBuildOptions(buildOpts, log, realFS)
 	finalizeBuildOptions(&options)
 	if buildOpts.AbsWorkingDir != absWorkingDir {
@@ -888,19 +934,33 @@ func contextImpl(buildOpts BuildOptions) (*internalContext, []Message) {
 
 	// If we have errors already, then refuse to build any further. This only
 	// happens when the build options themselves contain validation errors.
-	if msgs := log.Done(); log.HasErrors() {
+	msgs := log.Done()
+	if log.HasErrors() {
+		if logOptions.LogLevel < logger.LevelSilent {
+			// Print all deferred validation log messages to stderr. We defer all log
+			// messages that are generated above because warnings are re-printed for
+			// every rebuild and we don't want to double-print these warnings for the
+			// first build.
+			stderr := logger.NewStderrLog(logOptions)
+			for _, msg := range msgs {
+				stderr.AddMsg(msg)
+			}
+			stderr.Done()
+		}
 		return nil, convertMessagesToPublic(logger.Error, msgs)
 	}
 
 	args := rebuildArgs{
-		caches:         caches,
-		onEndCallbacks: onEndCallbacks,
-		logOptions:     logOptions,
-		entryPoints:    entryPoints,
-		options:        options,
-		mangleCache:    buildOpts.MangleCache,
-		absWorkingDir:  absWorkingDir,
-		write:          buildOpts.Write,
+		caches:             caches,
+		onEndCallbacks:     onEndCallbacks,
+		onDisposeCallbacks: onDisposeCallbacks,
+		logOptions:         logOptions,
+		logWarnings:        msgs,
+		entryPoints:        entryPoints,
+		options:            options,
+		mangleCache:        buildOpts.MangleCache,
+		absWorkingDir:      absWorkingDir,
+		write:              buildOpts.Write,
 	}
 
 	return &internalContext{
@@ -913,6 +973,7 @@ func contextImpl(buildOpts BuildOptions) (*internalContext, []Message) {
 type buildInProgress struct {
 	state     rebuildState
 	waitGroup sync.WaitGroup
+	cancel    config.CancelFlag
 }
 
 type internalContext struct {
@@ -952,6 +1013,7 @@ func (ctx *internalContext) rebuild() rebuildState {
 	watcher := ctx.watcher
 	handler := ctx.handler
 	oldSummary := ctx.latestSummary
+	args.options.CancelFlag = &build.cancel
 	ctx.mutex.Unlock()
 
 	// Do the build without holding the mutex
@@ -1066,6 +1128,27 @@ func (ctx *internalContext) Watch(options WatchOptions) error {
 	return nil
 }
 
+func (ctx *internalContext) Cancel() {
+	ctx.mutex.Lock()
+
+	// Ignore disposed contexts
+	if ctx.didDispose {
+		ctx.mutex.Unlock()
+		return
+	}
+
+	build := ctx.activeBuild
+	ctx.mutex.Unlock()
+
+	if build != nil {
+		// Tell observers to cut this build short
+		build.cancel.Cancel()
+
+		// Wait for the build to finish before returning
+		build.waitGroup.Wait()
+	}
+}
+
 func (ctx *internalContext) Dispose() {
 	// Only dispose once
 	ctx.mutex.Lock()
@@ -1093,6 +1176,11 @@ func (ctx *internalContext) Dispose() {
 	if build != nil {
 		build.waitGroup.Wait()
 	}
+
+	// Run each "OnDispose" callback on its own goroutine
+	for _, fn := range ctx.args.onDisposeCallbacks {
+		go fn()
+	}
 }
 
 func prettyPrintByteCount(n int) string {
@@ -1345,14 +1433,16 @@ type onEndCallback struct {
 }
 
 type rebuildArgs struct {
-	caches         *cache.CacheSet
-	onEndCallbacks []onEndCallback
-	logOptions     logger.OutputOptions
-	entryPoints    []bundler.EntryPoint
-	options        config.Options
-	mangleCache    map[string]interface{}
-	absWorkingDir  string
-	write          bool
+	caches             *cache.CacheSet
+	onEndCallbacks     []onEndCallback
+	onDisposeCallbacks []func()
+	logOptions         logger.OutputOptions
+	logWarnings        []logger.Msg
+	entryPoints        []bundler.EntryPoint
+	options            config.Options
+	mangleCache        map[string]interface{}
+	absWorkingDir      string
+	write              bool
 }
 
 type rebuildState struct {
@@ -1365,6 +1455,11 @@ type rebuildState struct {
 func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 	log := logger.NewStderrLog(args.logOptions)
 
+	// All validation warnings are repeated for every rebuild
+	for _, msg := range args.logWarnings {
+		log.AddMsg(msg)
+	}
+
 	// Convert and validate the buildOpts
 	realFS, err := fs.RealFS(fs.RealFSOptions{
 		AbsWorkingDir: args.absWorkingDir,
@@ -1398,6 +1493,11 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 		result.MangleCache = cloneMangleCache(log, args.mangleCache)
 		results, metafile := bundle.Compile(log, timer, result.MangleCache, linker.Link)
 
+		// Canceling a build generates a single error at the end of the build
+		if args.options.CancelFlag.DidCancel() {
+			log.AddError(nil, logger.Range{}, "The build was canceled")
+		}
+
 		// Stop now if there were errors
 		if !log.HasErrors() {
 			result.Metafile = metafile
@@ -1492,7 +1592,10 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 	result.Errors = convertMessagesToPublic(logger.Error, msgs)
 	result.Warnings = convertMessagesToPublic(logger.Warning, msgs)
 
-	// Run any registered "OnEnd" callbacks now
+	// Run any registered "OnEnd" callbacks now. These always run regardless of
+	// whether the current build has bee canceled or not. They can check for
+	// errors by checking the error array in the build result, and canceled
+	// builds should always have at least one error.
 	timer.Begin("On-end callbacks")
 	for _, onEnd := range args.onEndCallbacks {
 		fromPlugin, thrown := onEnd.fn(&result)
@@ -1709,7 +1812,7 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		}
 
 		// Scan over the bundle
-		mockFS := fs.MockFS(make(map[string]string), fs.MockUnix)
+		mockFS := fs.MockFS(make(map[string]string), fs.MockUnix, "/")
 		bundle := bundler.ScanBundle(log, mockFS, caches, nil, options, timer)
 
 		// Stop now if there were errors
@@ -1935,6 +2038,7 @@ func (impl *pluginImpl) validatePathsArray(pathsIn []string, name string) (paths
 
 func loadPlugins(initialOptions *BuildOptions, fs fs.FS, log logger.Log, caches *cache.CacheSet) (
 	onEndCallbacks []onEndCallback,
+	onDisposeCallbacks []func(),
 	finalizeBuildOptions func(*config.Options),
 ) {
 	// Clone the plugin array to guard against mutation during iteration
@@ -2036,11 +2140,16 @@ func loadPlugins(initialOptions *BuildOptions, fs fs.FS, log logger.Log, caches
 			})
 		}
 
+		onDispose := func(fn func()) {
+			onDisposeCallbacks = append(onDisposeCallbacks, fn)
+		}
+
 		item.Setup(PluginBuild{
 			InitialOptions: initialOptions,
 			Resolve:        resolve,
 			OnStart:        impl.onStart,
 			OnEnd:          onEnd,
+			OnDispose:      onDispose,
 			OnResolve:      impl.onResolve,
 			OnLoad:         impl.onLoad,
 		})
@@ -2401,3 +2510,38 @@ func summarizeOutputFiles(outputFiles []OutputFile) buildSummary {
 	}
 	return summary
 }
+
+func stripDirPrefix(path string, prefix string, allowedSlashes string) (string, bool) {
+	if strings.HasPrefix(path, prefix) {
+		pathLen := len(path)
+		prefixLen := len(prefix)
+
+		// Just return the path if there is no prefix
+		if prefixLen == 0 {
+			return path, true
+		}
+
+		// Return the empty string if the path equals the prefix
+		if pathLen == prefixLen {
+			return "", true
+		}
+
+		if strings.IndexByte(allowedSlashes, prefix[prefixLen-1]) >= 0 {
+			// Return the suffix if the prefix ends in a slash. Examples:
+			//
+			//   stripDirPrefix(`/foo`, `/`, `/`) => `foo`
+			//   stripDirPrefix(`C:\foo`, `C:\`, `\/`) => `foo`
+			//
+			return path[prefixLen:], true
+		} else if strings.IndexByte(allowedSlashes, path[prefixLen]) >= 0 {
+			// Return the suffix if there's a slash after the prefix. Examples:
+			//
+			//   stripDirPrefix(`/foo/bar`, `/foo`, `/`) => `bar`
+			//   stripDirPrefix(`C:\foo\bar`, `C:\foo`, `\/`) => `bar`
+			//
+			return path[prefixLen+1:], true
+		}
+	}
+
+	return "", false
+}
